! ------------------------------------------------------------------------
!>  \brief Module for testing Banded Symmetric Matrix data type
! ------------------------------------------------------------------------
MODULE band_sym_matrix_test
USE fruit                   !> Unit testing framework
USE log_message_control     !> Printing log/error messages
USE log_messages            !> Log/error codes
USE band_sym_matrix_def     !> Banded Symmetric Matrix data type
USE dense_matrix_def        !> Dense Matrix data type
USE vector_def              !> Vector data type

IMPLICIT NONE


CONTAINS


! ------------------------------------------------------------------------
!>  \test Test for OK exception message on allocation
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test       Test banded symmetric matrix
!!  \param testName   Filename for log file (required for exceptions)
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!
!!  This test initializes a bandSymMatrixT and makes sure that the
!!  exception message is OK (i.e. allocation did not fail).
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_allocation_MSG
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_allocation_MSG'
  TYPE(bandSymMatrixT) :: test
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER :: expMsg=OK
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log message file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize matrix and check the exception
  CALL bsm_init(test, hbw,n, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize log file and deallocate matrix
  CALL log_closeLogFile()
  CALL bsm_clean(test)

END SUBROUTINE test_bsm_allocation_MSG


! ------------------------------------------------------------------------
!>  \test Test for SZE exception message on allocation
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test     Test banded symmetric matrix
!!  \param testName Filename for log file (required for exceptions)
!!  \param hbw      Half bandwidth of the matrix
!!  \param n        Number of rows/columns in the matrix
!!  \param expMsg   Expected log message
!!  \param actMsg   Actual log message
!!
!!  This test attempts to initialize a bandSymMatrixT with invalid size
!!  parameters and verifies that the correct exception is returned.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_allocation_SZE
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_allocation_SZE'
  TYPE(bandSymMatrixT) :: test
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER :: expMsg=SZE
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> try to initialize with hbw=-1
  CALL bsm_init(test, -1,n, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to initialize with n=-1
  CALL bsm_init(test, hbw,-1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize log file and deallocate matrix
  CALL log_closeLogFile()
  CALL bsm_clean(test)

END SUBROUTINE test_bsm_allocation_SZE


! ------------------------------------------------------------------------
!>  \test Test for initialization of matrix data to zeros.
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test     Test banded symmetric matrix
!!  \param hbw      Half bandwidth of the matrix
!!  \param n        Number of rows/columns in the matrix
!!  \param expDat   Expected initial contents of matrix data
!!
!!  This test initializes a bandSymMatrixT and ensures that the data is
!!  initialized to zeros.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_allocation_DAT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_allocation_DAT'
  TYPE(bandSymMatrixT) :: test
  INTEGER, PARAMETER :: hbw=3,n=10
  DOUBLE PRECISION, DIMENSION(hbw,n) :: expDat

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> set up the expected data (note: Fortran uses column-major storage)
  expDat = RESHAPE( (/ &
                    0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0, &
                    0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0, &
                    0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0  &
                    /), SHAPE(expDat) )

  !> initialize matrix and test actual data
  CALL bsm_init(test, hbw,n)
  CALL assertEquals(expDat,test%dat, hbw,n)
  CALL assertEquals(expDat,test%decomp, hbw,n)

  !> deallocate matrix
  CALL bsm_clean(test)

END SUBROUTINE test_bsm_allocation_DAT


! ------------------------------------------------------------------------
!>  \test Test for deallocation
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test       Test banded symmetric matrix
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expBefore  Expected allocation status before deallocation
!!  \param expAfter   Expected allocation status after deallocation
!!  \param actBefore  Actual allocation status before deallocation
!!  \param actAfter   Actual allocation status after deallocation
!!
!!  This test initializes a bandSymMatrixT and ensures that it is
!!  allocated. It then deallocates the object and ensures that it has
!!  been deallocated.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_deallocation
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_deallocation'
  TYPE(bandSymMatrixT) :: test
  INTEGER, PARAMETER :: hbw=3,n=10
  LOGICAL, PARAMETER :: expBefore = .TRUE.
  LOGICAL, PARAMETER :: expAfter = .FALSE.
  LOGICAL :: actBefore, actAfter

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize the matrix
  CALL bsm_init(test, hbw,n)

  !> make sure dat is allocated
  actBefore = ALLOCATED(test%dat)
  CALL assertEquals(expBefore,actBefore)

  !> make sure decomp is allocated
  actBefore = ALLOCATED(test%decomp)
  CALL assertEquals(expBefore,actBefore)

  !> deallocate the matrix
  CALL bsm_clean(test)

  !> make sure dat is deallocated
  actAfter = ALLOCATED(test%dat)
  CALL assertEquals(expAfter,actAfter)

  !> make sure decomp is deallocated
  actAfter = ALLOCATED(test%decomp)
  CALL assertEquals(expAfter,actAfter)

END SUBROUTINE test_bsm_deallocation


! ------------------------------------------------------------------------
!>  \test Test for number of rows when matrix is not allocated
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test       Test banded symmetric matrix
!!  \param expected   Expected number of rows
!!  \param actual     Actual number of rows
!!
!!  This test makes sure that the number of rows is returned as 0 when
!!  the matrix is not allocated
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_num_rows_not_allocated
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_num_rows_not_allocated'
  TYPE(bandSymMatrixT) :: test
  INTEGER, PARAMETER :: expected = 0
  INTEGER :: actual

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> check number of rows
  actual = bsm_numRows(test)
  CALL assertEquals(expected, actual)

END SUBROUTINE test_bsm_num_rows_not_allocated


! ------------------------------------------------------------------------
!>  \test Test for number of rows when matrix is allocated
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test       Test banded symmetric matrix
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expected   Expected number of rows
!!  \param actual     Actual number of rows
!!
!!  This test checks that the number of rows returned is correct when the
!!  matrix is allocated.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_num_rows_allocated
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_num_rows_allocated'
  TYPE(bandSymMatrixT) :: test
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER :: expected = n
  INTEGER :: actual

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize the matrix
  CALL bsm_init(test, hbw,n)

  !> check number of rows
  actual = bsm_numRows(test)
  CALL assertEquals(expected, actual)

  !> deallocate the matrix
  CALL bsm_clean(test)

END SUBROUTINE test_bsm_num_rows_allocated


! ------------------------------------------------------------------------
!>  \test Test for half bandwidth when matrix is not allocated
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test       Test banded symmetric matrix
!!  \param expected   Expected number of rows
!!  \param actual     Actual number of rows
!!
!!  This test makes sure that the half bandwidth is returned as 0 when
!!  the matrix is not allocated
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_half_bw_not_allocated
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_half_bw_not_allocated'
  TYPE(bandSymMatrixT) :: test
  INTEGER, PARAMETER :: expected = 0
  INTEGER :: actual

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> check the half bandwidth
  actual = bsm_halfBW(test)
  CALL assertEquals(expected, actual)

END SUBROUTINE test_bsm_half_bw_not_allocated


! ------------------------------------------------------------------------
!>  \test Test for half bandwidth when matrix is allocated
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test       Test banded symmetric matrix
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expected   Expected number of rows
!!  \param actual     Actual number of rows
!!
!!  This test checks that the half bandwidth returned is correct when the
!!  matrix is allocated.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_half_bw_allocated
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_half_bw_allocated'
  TYPE(bandSymMatrixT) :: test
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER :: expected = hbw
  INTEGER :: actual

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize the matrix
  CALL bsm_init(test, hbw,n)

  !> check the half bandwidth
  actual = bsm_halfBW(test)
  CALL assertEquals(expected, actual)

  !> deallocate the matrix
  CALL bsm_clean(test)

END SUBROUTINE test_bsm_half_bw_allocated


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from bsm_get
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test     Test banded symmetric matrix
!!  \param testName Filename for log file (required for exceptions)
!!  \param hbw      Half bandwidth of the matrix
!!  \param n        Number of rows/columns in the matrix
!!  \param expMsg   Expected log message
!!  \param actMsg   Actual log message
!!  \param v        Dummy variable for get function return
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the matrix.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_get_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_get_POSIT'
  TYPE(bandSymMatrixT) :: test
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION :: v

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize matrix
  CALL bsm_init(test, hbw,n)

  !> try to get beyond last row
  v = bsm_get(test, n+1,n, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first row
  v = bsm_get(test, 0,n, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get after last column
  v = bsm_get(test, n,n+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first column
  v = bsm_get(test, n,0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the matrix
  CALL log_closeLogFile()
  CALL bsm_clean(test)

END SUBROUTINE test_bsm_get_POSIT


! ------------------------------------------------------------------------
!>  \test Test for correct value return from bsm_get
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test     Test banded symmetric matrix
!!  \param hbw      Half bandwidth of the matrix
!!  \param n        Number of rows/columns in the matrix
!!  \param i        Row index for test location
!!  \param j        Column index for test location
!!  \param expVal   Expected data value
!!  \param actVal   Actual data value
!!
!!  This test checks that the correct value is returned from the get
!!  access program (i.e. that the location is interpreted correctly from
!!  the unpacked indices).
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_get_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_get_VAL'
  TYPE(bandSymMatrixT) :: test
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER :: i=3,j=5
  DOUBLE PRECISION, PARAMETER :: expVal = 3.d0
  DOUBLE PRECISION :: actVal

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize the matrix
  CALL bsm_init(test, hbw,n)

  !> set the value manually in the internal data structure
  test%dat(hbw+i-j,j) = expVal

  !> get the value using the access program
  actVal = bsm_get(test, i,j)
  CALL assertEquals(expVal,actVal)

  !> get the value using the access program (testing symmetry)
  actVal = bsm_get(test, j,i)
  CALL assertEquals(expVal,actVal)

  !> deallocate the matrix
  CALL bsm_clean(test)

END SUBROUTINE test_bsm_get_VAL


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from bsm_set
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test     Test banded symmetric matrix
!!  \param testName Filename for log file (required for exceptions)
!!  \param hbw      Half bandwidth of the matrix
!!  \param n        Number of rows/columns in the matrix
!!  \param expMsg   Expected log message
!!  \param actMsg   Actual log message
!!  \param v        Dummy variable for set function input
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the matrix.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_set_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_set_POSIT'
  TYPE(bandSymMatrixT) :: test
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER :: expMsg = POSIT
  DOUBLE PRECISION, PARAMETER :: v=0.d0
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize the log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize the matrix
  CALL bsm_init(test, hbw,n)

  !> try to set after last row
  CALL bsm_set(test, n+1,n,v, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first row
  CALL bsm_set(test, 0,n,v, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set after last column
  CALL bsm_set(test, n,n+1,v, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first column
  CALL bsm_set(test, n,0,v, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize log file and deallocate matrix
  CALL log_closeLogFile()
  CALL bsm_clean(test)

END SUBROUTINE test_bsm_set_POSIT


! ------------------------------------------------------------------------
!>  \test Test for correct value return after bsm_set
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test     Test banded symmetric matrix
!!  \param hbw      Half bandwidth of the matrix
!!  \param n        Number of rows/columns in the matrix
!!  \param i        Row index for test location
!!  \param j        Column index for test location
!!  \param expVal   Expected data value
!!  \param actVal   Actual data value
!!  \param reset    For resetting data value before symmetry test
!!
!!  This test checks that the correct value is entered by the set
!!  access program (i.e. that the location is interpreted correctly from
!!  the unpacked indices).
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_set_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_set_VAL'
  TYPE(bandSymMatrixT) :: test
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER :: i=3,j=5
  DOUBLE PRECISION, PARAMETER :: expVal = 3.d0
  DOUBLE PRECISION, PARAMETER :: reset = 0.d0
  DOUBLE PRECISION :: actVal

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize the matrix
  CALL bsm_init(test, hbw,n)

  !> set the value
  CALL bsm_set(test, i,j,expVal)
  actVal = bsm_get(test, i,j)
  CALL assertEquals(expVal,actVal)

  !> set the value (symmetry)
  CALL bsm_set(test, i,j,reset)
  CALL bsm_set(test, j,i,expVal)
  actVal = bsm_get(test, i,j)
  CALL assertEquals(expVal,actVal)

  !> deallocate matrix
  CALL bsm_clean(test)

END SUBROUTINE test_bsm_set_VAL


! ------------------------------------------------------------------------
!>  \test Test for DIMEN exception on setting decomposed matrix data
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test     Test banded symmetric matrix
!!  \param testName Filename for log file (required for exceptions)
!!  \param hbw      Half bandwidth of the matrix
!!  \param n        Number of rows/columns in the matrix
!!  \param expMsg   Expected log message
!!  \param actMsg   Actual log message
!!  \param decomp   Dummy data that has incorrect dimensions
!!
!!  This test checks that a DIMEN exception is returned when the
!!  decomposed data has incorrect dimensions. That is, not (hbw,n).
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_set_decomp_DIMEN
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_set_decomp_DIMEN'
  TYPE(bandSymMatrixT) :: test
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER :: expMsg = DIMEN
  DOUBLE PRECISION, DIMENSION(hbw-1,n) :: decomp
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> set up dummy data (note: Fortran uses column-major storage)
  decomp = RESHAPE( (/ &
                    0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0, &
                    0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0  &
                    /), SHAPE(decomp) )

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize matrix
  CALL bsm_init(test, hbw,n)

  !> try to set the decomposed data
  CALL bsm_setDecomp(test,decomp, actMsg)
  CALL assertEquals(expMsg, actMsg)

  !> finalize log file and deallocate matrix
  CALL log_closeLogFile()
  CALL bsm_clean(test)

END SUBROUTINE test_bsm_set_decomp_DIMEN


! ------------------------------------------------------------------------
!>  \test Test for correct setting of decomposed data
!!
!!  \param unit_name    Name of unit test (for FRUIT)
!!  \param test         Test banded symmetric matrix
!!  \param hbw          Half bandwidth of the matrix
!!  \param n            Number of rows/columns in the matrix
!!  \param expDecomp    Expected decomposed data
!!
!!  This test checks that the decomposed data is correctly set.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_set_decomp_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_set_decomp_VAL'
  TYPE(bandSymMatrixT) :: test
  INTEGER, PARAMETER :: hbw=3,n=10
  DOUBLE PRECISION, DIMENSION(hbw,n) :: expDecomp

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> set up decomposed data (note: Fortran uses column-major storage)
  expDecomp = RESHAPE((/ &
                      0.d0,0.d0,1.d0,0.d0,2.d0,3.d0,4.d0,5.d0,6.d0,7.d0, &
                      8.d0,9.d0,0.d0,1.d0,2.d0,3.d0,4.d0,5.d0,6.d0,7.d0, &
                      8.d0,9.d0,0.d0,1.d0,2.d0,3.d0,4.d0,5.d0,6.d0,7.d0  &
                      /), SHAPE(expDecomp) )

  !> initialize matrix
  CALL bsm_init(test, hbw,n)

  !> set the decomposed data
  CALL bsm_setDecomp(test,expDecomp)
  CALL assertEquals(expDecomp, test%decomp, hbw,n)

  !> deallocate matrix
  CALL bsm_clean(test)

END SUBROUTINE test_bsm_set_decomp_VAL


! ------------------------------------------------------------------------
!>  \test Test for correct status of switch for decomposed data
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test       Test banded symmetric matrix
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expBefore  Expected value of switch before setting decomp
!!  \param expAfter   Expected value of switch after setting decomp
!!  \param expClean   Expected value of switch after deallocating matrix
!!  \param actBefore  Actual value of switch before setting decomp
!!  \param actAfter   Actual value of switch after setting decomp
!!  \param actClean   Actual value of switch after deallocating matrix
!!  \param decomp     Dummy decomposed data
!!
!!  This test checks that the presence of decomposed data is correctly
!!  identified.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_is_decomposed
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_is_decomposed'
  TYPE(bandSymMatrixT) :: test
  INTEGER, PARAMETER :: hbw=3,n=10
  LOGICAL, PARAMETER :: expBefore = .FALSE.
  LOGICAL, PARAMETER :: expAfter = .TRUE.
  LOGICAL, PARAMETER :: expClean = .FALSE.
  DOUBLE PRECISION, DIMENSION(hbw,n) :: decomp
  LOGICAL :: actBefore, actAfter, actClean

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize matrix and check initial value of switch
  CALL bsm_init(test, hbw,n)
  actBefore = bsm_isDecomposed(test)
  CALL assertEquals(expBefore,actBefore)

  !> set up decomposed data (note: Fortran uses column-major storage)
  decomp = RESHAPE( (/ &
                    0.d0,0.d0,1.d0,0.d0,2.d0,3.d0,4.d0,5.d0,6.d0,7.d0, &
                    8.d0,9.d0,0.d0,1.d0,2.d0,3.d0,4.d0,5.d0,6.d0,7.d0, &
                    8.d0,9.d0,0.d0,1.d0,2.d0,3.d0,4.d0,5.d0,6.d0,7.d0  &
                    /), SHAPE(decomp) )

  !> set decomposed data
  CALL bsm_setDecomp(test,decomp)
  actAfter = bsm_isDecomposed(test)
  CALL assertEquals(expAfter,actAfter)

  !> deallocate matrix and check final value of decomposed switch
  CALL bsm_clean(test)
  actClean = bsm_isDecomposed(test)
  CALL assertEquals(expClean,actClean)

END SUBROUTINE test_bsm_is_decomposed


! ------------------------------------------------------------------------
!>  \test Test for DIMEN exception when adding matrices
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1      First input banded symmetric matrix
!!  \param test2      Second input banded symmetric matrix
!!  \param test3      Output banded symmetric matrix
!!  \param testName   Filename for log file (required for exceptions)
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expMsg     Expected exception message
!!  \param actMsg     Actual exception message
!!
!!  This test checks that a DIMEN exception is raised when matrices with
!!  incompatible dimensions are added.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_add_DIMEN
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_add_DIMEN'
  TYPE(bandSymMatrixT) :: test1, test2, test3
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER :: expMsg=DIMEN
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize input matrices (data contents not important)
  CALL bsm_init(test1, hbw,n)
  CALL bsm_init(test2, hbw,n+1) !> note different dimension

  !> attempt to add the matrices
  test3 = bsm_add(test1,test2, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize log file and deallocate matrices
  CALL log_closeLogFile()
  CALL bsm_clean(test1)
  CALL bsm_clean(test2)
  CALL bsm_clean(test3)

END SUBROUTINE test_bsm_add_DIMEN


! ------------------------------------------------------------------------
!>  \test Test for correct addition of matrices
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1      First input banded symmetric matrix
!!  \param test2      Second input banded symmetric matrix
!!  \param test3      Output banded symmetric matrix
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expDat     Expected result of addition
!!
!!  This test checks that the correct result is obtained when adding two
!!  banded symmetric matrices.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_add_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_add_VAL'
  TYPE(bandSymMatrixT) :: test1, test2, test3
  INTEGER, PARAMETER :: hbw=3,n=10
  DOUBLE PRECISION, DIMENSION(hbw,n) :: expDat
  INTEGER :: i,j  !> loop variables

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize expected result (note: Fortran uses column-major storage)
  expDat = RESHAPE( (/ &
                    0.d0,0.d0,3.d0,0.d0,3.d0,3.d0,1.d0,3.d0,3.d0,1.d0, &
                    3.d0,3.d0,1.d0,3.d0,3.d0,1.d0,3.d0,3.d0,1.d0,3.d0, &
                    3.d0,1.d0,3.d0,3.d0,1.d0,3.d0,3.d0,1.d0,3.d0,3.d0  &
                    /), SHAPE(expDat) )

  !> initialize first input matrix
  !! (loop sets matrix on bands to all ones)
  CALL bsm_init(test1, hbw,n)
  DO i = 1,n
    DO j = i,MIN(i+hbw-1,n)
      CALL bsm_set(test1, i,j, 1.d0)
    END DO
  END DO

  !> initialize second input matrix (note different bandwidth)
  !! (loop sets matrix on bands to all twos)
  CALL bsm_init(test2, hbw-1,n)
  DO i = 1,n
    DO j = i,MIN(i+(hbw-1)-1,n)
     CALL bsm_set(test2, i,j, 2.d0)
    END DO
  END DO

  !> add the matrices
  test3 = bsm_add(test1,test2)
  CALL assertEquals(expDat,test3%dat, hbw,n)

  !> deallocate matrices
  CALL bsm_clean(test1)
  CALL bsm_clean(test2)
  CALL bsm_clean(test3)

END SUBROUTINE test_bsm_add_VAL


! ------------------------------------------------------------------------
!>  \test Test for correct addition of matrices ( with OPERATOR (+) )
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1      First input banded symmetric matrix
!!  \param test2      Second input banded symmetric matrix
!!  \param test3      Output banded symmetric matrix
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expDat     Expected result of addition
!!
!!  This test checks that the correct result is obtained when adding two
!!  banded symmetric matrices using the (+) operator.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_add_OP
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_add_OP'
  TYPE(bandSymMatrixT) :: test1, test2, test3
  INTEGER, PARAMETER :: hbw=3,n=10
  DOUBLE PRECISION, DIMENSION(hbw,n) :: expDat
  INTEGER :: i,j  !> loop variables

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize expected result (note: Fortran uses column-major storage)
  expDat = RESHAPE( (/ &
                    0.d0,0.d0,3.d0,0.d0,3.d0,3.d0,1.d0,3.d0,3.d0,1.d0, &
                    3.d0,3.d0,1.d0,3.d0,3.d0,1.d0,3.d0,3.d0,1.d0,3.d0, &
                    3.d0,1.d0,3.d0,3.d0,1.d0,3.d0,3.d0,1.d0,3.d0,3.d0  &
                    /), SHAPE(expDat) )

  !> initialize first input matrix
  !! (loop sets matrix on bands to all ones)
  CALL bsm_init(test1, hbw,n)
  DO i = 1,n
    DO j = i,MIN(i+hbw-1,n)
      CALL bsm_set(test1, i,j, 1.d0)
    END DO
  END DO

  !> initialize second input matrix (note different bandwidth)
  !! (loop sets matrix on bands to all twos)
  CALL bsm_init(test2, hbw-1,n)
  DO i = 1,n
    DO j = i,MIN(i+(hbw-1)-1,n)
     CALL bsm_set(test2, i,j, 2.d0)
    END DO
  END DO

  !> add the matrices
  test3 = test1 + test2
  CALL assertEquals(expDat,test3%dat, hbw,n)

  !> deallocate matrices
  CALL bsm_clean(test1)
  CALL bsm_clean(test2)
  CALL bsm_clean(test3)

END SUBROUTINE test_bsm_add_OP


! ------------------------------------------------------------------------
!>  \test Test for DIMEN exception when performing mapped add
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1      Test banded symmetric matrix
!!  \param test2      Test input dense matrix
!!  \param testName   Filename for log file (required for exceptions)
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expMsg     Expected exception message
!!  \param actMsg     Actual exception message
!!  \param ind        Mapping indices (size does not match test2)
!!
!!  This test checks that a DIMEN exception is raised when mapped addition
!!  is attempted with the dimension of the index array not matching the
!!  dimensions of the dense matrix.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_mapped_add_DIMEN1
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_mapped_add_DIMEN1'
  TYPE(bandSymMatrixT) :: test1
  TYPE(matrixT) :: test2
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER :: expMsg=DIMEN
  INTEGER, PARAMETER, DIMENSION(hbw-1) :: ind = (/ 1,2 /)
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize matrices (data content not important)
  CALL bsm_init(test1, hbw,n)
  CALL dm_init(test2, hbw,hbw)

  !> try mapped add (note hbw=3, SIZE(ind)=2)
  CALL bsm_mappedAdd(test1, test2,ind, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize log file and deallocate matrices
  CALL log_closeLogFile()
  CALL bsm_clean(test1)
  CALL dm_clean(test2)

END SUBROUTINE test_bsm_mapped_add_DIMEN1


! ------------------------------------------------------------------------
!>  \test Test for DIMEN exception when performing mapped add
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1      Test banded symmetric matrix
!!  \param test2      Test input dense matrix (not square)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expMsg     Expected exception message
!!  \param actMsg     Actual exception message
!!  \param ind        Mapping indices
!!
!!  This test checks that a DIMEN exception is raised when mapped addition
!!  is attempted with a rectangular input dense matrix.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_mapped_add_DIMEN2
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_mapped_add_DIMEN2'
  TYPE(bandSymMatrixT) :: test1
  TYPE(matrixT) :: test2
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER :: expMsg=DIMEN
  INTEGER, PARAMETER, DIMENSION(hbw) :: ind = (/ 1,2,3 /)
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize matrices (note test2 is not square)
  CALL bsm_init(test1, hbw,n)
  CALL dm_init(test2, hbw+1,hbw)

  !> attempt mapped add
  CALL bsm_mappedAdd(test1, test2,ind, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize log file and deallocate matrices
  CALL log_closeLogFile()
  CALL bsm_clean(test1)
  CALL dm_clean(test2)

END SUBROUTINE test_bsm_mapped_add_DIMEN2


! ------------------------------------------------------------------------
!>  \test Test for DIMEN exception when performing mapped add
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1      Test banded symmetric matrix
!!  \param test2      Test input dense matrix (larger than hbw)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expMsg     Expected exception message
!!  \param actMsg     Actual exception message
!!  \param ind        Mapping indices (larger than hbw)
!!
!!  This test checks that a DIMEN exception is raised when mapped addition
!!  is attempted with an input dense matrix whose size exceeds the half
!!  bandwidth of the banded symmetric matrix.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_mapped_add_DIMEN3
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_mapped_add_DIMEN3'
  TYPE(bandSymMatrixT) :: test1
  TYPE(matrixT) :: test2
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER :: expMsg=DIMEN
  INTEGER, PARAMETER, DIMENSION(hbw+1) :: ind = (/ 1,2,3,4 /)
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize matrices (note that test2 is wider than hbw)
  CALL bsm_init(test1, hbw,n)
  CALL dm_init(test2, hbw+1,hbw+1)

  !> attempt mapped add
  CALL bsm_mappedAdd(test1, test2,ind, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize log file and deallocate matrices
  CALL log_closeLogFile()
  CALL bsm_clean(test1)
  CALL dm_clean(test2)

END SUBROUTINE test_bsm_mapped_add_DIMEN3


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception when performing mapped add
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1      Test banded symmetric matrix
!!  \param test2      Test input dense matrix
!!  \param testName   Filename for log file (required for exceptions)
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expMsg     Expected exception message
!!  \param actMsg     Actual exception message
!!  \param ind1       Mapping indices (with index below minimum)
!!  \param ind2       Mapping indices (with index above maximum)
!!
!!  This test checks that a POSIT exception is raised when mapped addition
!!  is attempted with mapping indices that are outside the matrix.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_mapped_add_POSIT1
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_mapped_add_POSIT1'
  TYPE(bandSymMatrixT) :: test1
  TYPE(matrixT) :: test2
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER :: expMsg=POSIT
  INTEGER, PARAMETER, DIMENSION(hbw) :: ind1 = (/ -1,0,1 /)
  INTEGER, PARAMETER, DIMENSION(hbw) :: ind2 = (/ 9,10,11 /)
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize matrices (data not important)
  CALL bsm_init(test1, hbw,n)
  CALL dm_init(test2, hbw,hbw)

  !> attempt first mapped add
  CALL bsm_mappedAdd(test1, test2,ind1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> attempt second mapped add
  CALL bsm_mappedAdd(test1, test2,ind2, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize log file and deallocate matrices
  CALL log_closeLogFile()
  CALL bsm_clean(test1)
  CALL dm_clean(test2)

END SUBROUTINE test_bsm_mapped_add_POSIT1


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception when performing mapped add
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1      Test banded symmetric matrix
!!  \param test2      Test input dense matrix
!!  \param testName   Filename for log file (required for exceptions)
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expMsg     Expected exception message
!!  \param actMsg     Actual exception message
!!  \param ind        Mapping indices (range of indices >= half bandwidth)
!!
!!  This test checks that a POSIT exception is raised when mapped addition
!!  is attempted with mapping indices that have a difference equal to the
!!  half bandwidth (i.e. trying to set a value outside the band).
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_mapped_add_POSIT2
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_mapped_add_POSIT2'
  TYPE(bandSymMatrixT) :: test1
  TYPE(matrixT) :: test2
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER :: expMsg=POSIT
  INTEGER, PARAMETER, DIMENSION(hbw) :: ind = (/ 4,6,9 /)
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize matrices (data content not important)
  CALL bsm_init(test1, hbw,n)
  CALL dm_init(test2, hbw,hbw)

  !> attempt mapped add
  CALL bsm_mappedAdd(test1, test2,ind, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize log file and deallocate matrices
  CALL log_closeLogFile()
  CALL bsm_clean(test1)
  CALL dm_clean(test2)

END SUBROUTINE test_bsm_mapped_add_POSIT2


! ------------------------------------------------------------------------
!>  \test Test for correct mapped addition (single operation)
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1      Test banded symmetric matrix
!!  \param test2      Test input dense matrix
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expDat     Expected data contents
!!  \param ind        Mapping indices
!!
!!  This test checks that mapped addition is performed correctly. The
!!  operation is performed once on a blank matrix.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_mapped_add_VAL1
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_mapped_add_VAL1'
  TYPE(bandSymMatrixT) :: test1
  TYPE(matrixT) :: test2
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER, DIMENSION(2) :: ind = (/ 1,3 /)
  DOUBLE PRECISION, DIMENSION(hbw,n) :: expDat
  INTEGER :: i,j    !> loop variables

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> set expected result of mapped addition (note: Fortran uses column-major storage)
  expDat = RESHAPE( (/ &
                    0.d0,0.d0,1.d0,0.d0,0.d0,0.d0,2.d0,0.d0,3.d0,0.d0, &
                    0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0, &
                    0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0  &
                    /), SHAPE(expDat) )

  !> initialize target matrix
  CALL bsm_init(test1, hbw,n)

  !> initialize input matrix
  !! (loop sets the data contents)
  CALL dm_init(test2, 2,2)
  DO j = 1,2
    DO i = 1,2
      CALL dm_set(test2, i,j, ( (DBLE(j)-1.d0) + DBLE(i) ) )
    END DO
  END DO

  !> perform mapped addition
  CALL bsm_mappedAdd(test1, test2,ind)
  CALL assertEquals(expDat,test1%dat, hbw,n)

  !> deallocate matrices
  CALL bsm_clean(test1)
  CALL dm_clean(test2)

END SUBROUTINE test_bsm_mapped_add_VAL1


! ------------------------------------------------------------------------
!>  \test Test for correct mapped addition (zero index)
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1      Test banded symmetric matrix
!!  \param test2      Test input dense matrix
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expDat     Expected data contents
!!  \param ind        Mapping indices (with zero index indicating no-op)
!!
!!  This test checks that mapped addition is performed correctly. The
!!  mapping index contains a zero, which should tell the mapped adder to
!!  skip that location in the input matrix.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_mapped_add_VAL2
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_mapped_add_VAL2'
  TYPE(bandSymMatrixT) :: test1
  TYPE(matrixT) :: test2
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER, DIMENSION(3) :: ind = (/ 1,0,3 /)
  DOUBLE PRECISION, DIMENSION(hbw,n) :: expDat
  INTEGER :: i,j    !> loop variables

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> set up expected result (note: Fortran uses column-major storage)
  expDat = RESHAPE( (/ &
                    0.d0,0.d0,1.d0,0.d0,0.d0,0.d0,3.d0,0.d0,5.d0,0.d0, &
                    0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0, &
                    0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0  &
                    /), SHAPE(expDat) )

  !> initialize target matrix
  CALL bsm_init(test1, hbw,n)

  !> initialize input matrix
  !! (loop sets the input data)
  CALL dm_init(test2, 3,3)
  DO j = 1,3
    DO i = 1,3
      CALL dm_set(test2, i,j, ( (DBLE(j)-1.d0) + DBLE(i) ) )
    END DO
  END DO

  !> perform mapped addition
  CALL bsm_mappedAdd(test1, test2,ind)
  CALL assertEquals(expDat,test1%dat, hbw,n)

  !> deallocate matrices
  CALL bsm_clean(test1)
  CALL dm_clean(test2)

END SUBROUTINE test_bsm_mapped_add_VAL2


! ------------------------------------------------------------------------
!>  \test Test for correct mapped addition (multiple operations)
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1      Test banded symmetric matrix
!!  \param test2      First test input dense matrix
!!  \param test3      Second test input dense matrix
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expDat     Expected data contents
!!  \param ind2       Mapping indices for test2
!!  \param ind3       Mapping indices for test3
!!
!!  This test checks that mapped addition is performed correctly. The
!!  operation is performed twice to ensure summation of overlapping
!!  values. That is, input is added to existing data in the target matrix
!!  rather than overwriting.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_mapped_add_VAL3
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_mapped_add_VAL3'
  TYPE(bandSymMatrixT) :: test1
  TYPE(matrixT) :: test2,test3
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER, DIMENSION(2) :: ind2 = (/ 1,3 /)
  INTEGER, PARAMETER, DIMENSION(3) :: ind3 = (/ 1,0,3 /)
  DOUBLE PRECISION, DIMENSION(hbw,n) :: expDat
  INTEGER :: i,j    !> loop variables

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> set up expected result (note: Fortran uses column-major storage)
  expDat = RESHAPE( (/ &
                    0.d0,0.d0,2.d0,0.d0,0.d0,0.d0,5.d0,0.d0,8.d0,0.d0, &
                    0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0, &
                    0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0  &
                    /), SHAPE(expDat) )

  !> initialize target matrix
  CALL bsm_init(test1, hbw,n)

  !> initialize first input matrix
  !! (loop sets up the data)
  CALL dm_init(test2, 2,2)
  DO j = 1,2
    DO i = 1,2
      CALL dm_set(test2, i,j, ( (DBLE(j)-1.d0) + DBLE(i) ) )
    END DO
  END DO

  !> initialize second input matrix
  !! (loop sets up the data)
  CALL dm_init(test3, 3,3)
  DO j = 1,3
    DO i = 1,3
      CALL dm_set(test3, i,j, ( (DBLE(j)-1.d0) + DBLE(i) ) )
    END DO
  END DO

  !> perform the mapped addition for both inputs
  CALL bsm_mappedAdd(test1, test2,ind2)
  CALL bsm_mappedAdd(test1, test3,ind3)
  CALL assertEquals(expDat,test1%dat, hbw,n)

  !> deallocate matrices
  CALL bsm_clean(test1)
  CALL dm_clean(test2)
  CALL dm_clean(test3)

END SUBROUTINE test_bsm_mapped_add_VAL3


! ------------------------------------------------------------------------
!>  \test Test for scalar multiplication
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1    Input banded symmetric matrix
!!  \param test2    Output banded symmetric matrix
!!  \param k        Scalar coefficient
!!  \param hbw      Half bandwidth of the matrix
!!  \param n        Number of rows/columns in the matrix
!!  \param expDat   Expected data contents
!!
!!  This test checks that scalar multiplication is performed correctly.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_scal_mul_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_scal_mul_VAL'
  TYPE(bandSymMatrixT) :: test1, test2
  DOUBLE PRECISION, PARAMETER :: k=3.d0
  INTEGER, PARAMETER :: hbw=3,n=10
  DOUBLE PRECISION, DIMENSION(hbw,n) :: expDat
  INTEGER :: i,j    !> loop variable

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> set up expected result (note: Fortran uses column-major storage)
  expDat = RESHAPE( (/ &
                    0.d0,0.d0,3.d0,0.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0, &
                    3.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0, &
                    3.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0  &
                    /), SHAPE(expDat) )

  !> initialize matrix
  !! (loop sets band to ones)
  CALL bsm_init(test1, hbw,n)
  DO i = 1,n
    DO j = i,MIN(i+hbw-1,n)
      CALL bsm_set(test1, i,j, 1.d0)
    END DO
  END DO

  !> multiply by scalar
  test2 = bsm_scalMul(test1,k)
  CALL assertEquals(expDat,test2%dat, hbw,n)

  !> deallocate matrices
  CALL bsm_clean(test1)
  CALL bsm_clean(test2)

END SUBROUTINE test_bsm_scal_mul_VAL


! ------------------------------------------------------------------------
!>  \test Test for scalar multiplication (by zero)
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1    Input banded symmetric matrix
!!  \param test2    Output banded symmetric matrix
!!  \param k        Scalar coefficient (zero)
!!  \param hbw      Half bandwidth of the matrix
!!  \param n        Number of rows/columns in the matrix
!!  \param expDat   Expected data contents
!!
!!  This test checks that scalar multiplication results in a zero matrix
!!  when the scalar factor is zero.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_scal_mul_ZERO
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_scal_mul_ZERO'
  TYPE(bandSymMatrixT) :: test1, test2
  DOUBLE PRECISION, PARAMETER :: k=0.d0
  INTEGER, PARAMETER :: hbw=3,n=10
  DOUBLE PRECISION, DIMENSION(hbw,n) :: expDat
  INTEGER :: i,j    !> loop variables

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> set up expected data (note: Fortran uses column-major storage)
  expDat = RESHAPE( (/ &
                    0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0, &
                    0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0, &
                    0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0  &
                    /), SHAPE(expDat) )

  !> initialize matrix
  !! (loop sets band to ones)
  CALL bsm_init(test1, hbw,n)
  DO i = 1,n
    DO j = i,MIN(i+hbw-1,n)
      CALL bsm_set(test1, i,j, 1.d0)
    END DO
  END DO

  !> multiply by zero
  test2 = bsm_scalMul(test1,k)
  CALL assertEquals(expDat,test2%dat, hbw,n)

  !> deallocate matrices
  CALL bsm_clean(test1)
  CALL bsm_clean(test2)

END SUBROUTINE test_bsm_scal_mul_ZERO


! ------------------------------------------------------------------------
!>  \test Test for scalar multiplication ( using OPERATOR (*) )
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1    Input banded symmetric matrix
!!  \param test2    Output banded symmetric matrix
!!  \param k        Scalar coefficient
!!  \param hbw      Half bandwidth of the matrix
!!  \param n        Number of rows/columns in the matrix
!!  \param expDat   Expected data contents
!!
!!  This test checks that scalar multiplication is performed correctly
!!  when using the overloaded (*) operator.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_scal_mul_OP
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_scal_mul_OP'
  TYPE(bandSymMatrixT) :: test1, test2
  DOUBLE PRECISION, PARAMETER :: k=3.d0
  INTEGER, PARAMETER :: hbw=3,n=10
  DOUBLE PRECISION, DIMENSION(hbw,n) :: expDat
  INTEGER :: i,j    !> loop variables

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> set up expected result (note: Fortran uses column-major storage)
  expDat = RESHAPE( (/ &
                    0.d0,0.d0,3.d0,0.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0, &
                    3.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0, &
                    3.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0,3.d0  &
                    /), SHAPE(expDat) )

  !> initialize input matrix
  !! (loop sets bands to ones)
  CALL bsm_init(test1, hbw,n)
  DO i = 1,n
    DO j = i,MIN(i+hbw-1,n)
      CALL bsm_set(test1, i,j, 1.d0)
    END DO
  END DO

  !> perform scalar multiplication
  test2 = k * test1
  CALL assertEquals(expDat,test2%dat, hbw,n)

  !> clear output matrix
  CALL bsm_clean(test2)

  !> perform scalar multiplication (switch order of inputs)
  test2 = test1 * k
  CALL assertEquals(expDat,test2%dat, hbw,n)

  !> deallocate matrices
  CALL bsm_clean(test1)
  CALL bsm_clean(test2)

END SUBROUTINE test_bsm_scal_mul_OP


! ------------------------------------------------------------------------
!>  \test Test for DIMEN exception when performing matrix-vector multiplication
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1      Input banded symmetric matrix
!!  \param test2      Input vector
!!  \param test3      Dummy output vector
!!  \param testName   Filename for log file (required for exceptions)
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expMsg     Expected exception message
!!  \param actMsg     Actual exception message
!!
!!  This test checks that a DIMEN exception is raised when matrix-vector
!!  multiplication is attempted with a vector whose length does not equal
!!  the number of rows in the matrix.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_vec_mul_DIMEN
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_vec_mul_DIMEN'
  TYPE(bandSymMatrixT) :: test1
  TYPE(vectorT) :: test2, test3
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: hbw=3,n=10
  INTEGER, PARAMETER :: expMsg=DIMEN
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> set up matrix and vector (data contents not important)
  CALL bsm_init(test1, hbw,n)
  CALL vec_init(test2, n+1)   !> note different size

  !> attempt matrix-vector multiplication
  test3 = bsm_vecMul(test1,test2, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize log file and deallocate objects
  CALL log_closeLogFile()
  CALL bsm_clean(test1)
  CALL vec_clean(test2)
  CALL vec_clean(test3)

END SUBROUTINE test_bsm_vec_mul_DIMEN


! ------------------------------------------------------------------------
!>  \test Test for correct matrix-vector multiplication
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1      Input banded symmetric matrix
!!  \param test2      Input vector
!!  \param test3      Output vector
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expDat     Expected result
!!
!!  This test checks that matrix-vector multiplication is performed
!!  correctly in the general case (input matrix and vector contain data,
!!  but it is not of a special form).
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_vec_mul_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_vec_mul_VAL'
  TYPE(bandSymMatrixT) :: test1
  TYPE(vectorT) :: test2, test3
  INTEGER, PARAMETER :: hbw=3,n=10
  DOUBLE PRECISION, DIMENSION(n) :: expDat
  INTEGER :: i,j    !> loop variables

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> set expected result (note: Fortran uses column-major storage)
  expDat = RESHAPE( (/ &
                    14.d0,  40.d0,  85.d0,  150.d0, 235.d0, &
                    340.d0, 465.d0, 610.d0, 566.d0, 488.d0  &
                    /), SHAPE(expDat) )

  !> initialize input matrix
  !! (loop sets up data)
  CALL bsm_init(test1, hbw,n)
  DO i = 1,n
    DO j = i,MIN(i+hbw-1,n)
      CALL bsm_set(test1, i,j, ( (DBLE(j)-1.d0) + DBLE(i) ) )
    END DO
  END DO

  !> initialize input vector
  !! (loop sets up data)
  CALL vec_init(test2, n)
  DO i = 1,n
    CALL vec_set(test2, i, DBLE(i))
  END DO

  !> perform multiplication
  test3 = bsm_vecMul(test1,test2)
  CALL assertEquals(expDat,test3%dat, n)

  !> deallocate objects
  CALL bsm_clean(test1)
  CALL vec_clean(test2)
  CALL vec_clean(test3)

END SUBROUTINE test_bsm_vec_mul_VAL


! ------------------------------------------------------------------------
!>  \test Test for matrix-vector multiplication (by the zero vector)
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1      Input banded symmetric matrix
!!  \param test2      Input vector (zeros)
!!  \param test3      Output vector
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expDat     Expected result
!!
!!  This test checks that matrix-vector multiplication is performed
!!  correctly when the input vector is all zeros.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_vec_mul_ZERO
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_vec_mul_ZERO'
  TYPE(bandSymMatrixT) :: test1
  TYPE(vectorT) :: test2, test3
  INTEGER, PARAMETER :: hbw=3,n=10
  DOUBLE PRECISION, DIMENSION(n) :: expDat
  INTEGER :: i,j    !> loop variables

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> set up expected result (note: Fortran uses column-major storage)
  expDat = RESHAPE( (/ &
                    0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0 &
                    /), SHAPE(expDat) )

  !> initialize input matrix
  !! (loop sets up data)
  CALL bsm_init(test1, hbw,n)
  DO i = 1,n
    DO j = i,MIN(i+hbw-1,n)
      CALL bsm_set(test1, i,j, ( (DBLE(j)-1.d0) + DBLE(i) ) )
    END DO
  END DO

  !> initialize input vector (all zeros)
  CALL vec_init(test2, n)

  !> perform multiplication
  test3 = bsm_vecMul(test1,test2)
  CALL assertEquals(expDat,test3%dat, n)

  !> deallocate objects
  CALL bsm_clean(test1)
  CALL vec_clean(test2)
  CALL vec_clean(test3)

END SUBROUTINE test_bsm_vec_mul_ZERO


! ------------------------------------------------------------------------
!>  \test Test for matrix-vector multiplication (by the identity matrix)
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1      Input banded symmetric matrix (identity)
!!  \param test2      Input vector
!!  \param test3      Output vector
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expDat     Expected result
!!
!!  This test checks that matrix-vector multiplication is performed
!!  correctly when the input matrix is the identity for multiplication.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_vec_mul_IDENT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_vec_mul_IDENT'
  TYPE(bandSymMatrixT) :: test1
  TYPE(vectorT) :: test2, test3
  INTEGER, PARAMETER :: hbw=3,n=10
  DOUBLE PRECISION, DIMENSION(n) :: expDat
  INTEGER :: i,j    !> loop variables

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize input matrix
  !! (loop sets up identity matrix)
  CALL bsm_init(test1, hbw,n)
  DO i = 1,n
    CALL bsm_set(test1, i,i, 1.d0)
  END DO

  !> initialize input vector
  !! (loop sets data)
  CALL vec_init(test2, n)
  DO i = 1,n
    CALL vec_set(test2, i, DBLE(i))
  END DO

  !> expected result is the same as the initial vector
  expDat = test2%dat

  !> perform multiplication
  test3 = bsm_vecMul(test1,test2)
  CALL assertEquals(expDat,test3%dat, n)

  !> deallocate objects
  CALL bsm_clean(test1)
  CALL vec_clean(test2)
  CALL vec_clean(test3)

END SUBROUTINE test_bsm_vec_mul_IDENT


! ------------------------------------------------------------------------
!>  \test Test for correct matrix-vector multiplication ( using OPERATOR (*) )
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param test1      Input banded symmetric matrix
!!  \param test2      Input vector
!!  \param test3      Output vector
!!  \param hbw        Half bandwidth of the matrix
!!  \param n          Number of rows/columns in the matrix
!!  \param expDat     Expected result
!!
!!  This test checks that matrix-vector multiplication is performed
!!  correctly in the general case (input matrix and vector contain data,
!!  but it is not of a special form). The overloaded operator (*) is used.
! ------------------------------------------------------------------------
SUBROUTINE test_bsm_vec_mul_OP
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_bsm_vec_mul_OP'
  TYPE(bandSymMatrixT) :: test1
  TYPE(vectorT) :: test2, test3
  INTEGER, PARAMETER :: hbw=3,n=10
  DOUBLE PRECISION, DIMENSION(n) :: expDat
  INTEGER :: i,j    !> loop variables

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> set up expected result (note: Fortran uses column-major storage)
  expDat = RESHAPE( (/ &
                    14.d0,  40.d0,  85.d0,  150.d0, 235.d0, &
                    340.d0, 465.d0, 610.d0, 566.d0, 488.d0  &
                    /), SHAPE(expDat) )

  !> initialize input matrix
  !! (loop sets data)
  CALL bsm_init(test1, hbw,n)
  DO i = 1,n
    DO j = i,MIN(i+hbw-1,n)
      CALL bsm_set(test1, i,j, ( (DBLE(j)-1.d0) + DBLE(i) ) )
    END DO
  END DO

  !> initialize input vector
  !! (loop sets data)
  CALL vec_init(test2, n)
  DO i = 1,n
    CALL vec_set(test2, i, DBLE(i))
  END DO

  !> perform multiplication
  test3 = test1 * test2
  CALL assertEquals(expDat,test3%dat, n)

  !> deallocate objects
  CALL bsm_clean(test1)
  CALL vec_clean(test2)
  CALL vec_clean(test3)

END SUBROUTINE test_bsm_vec_mul_OP



END MODULE band_sym_matrix_test
