! ------------------------------------------------------------------------
!>  \brief Module for testing Field Data module
! ------------------------------------------------------------------------
MODULE field_data_test
USE fruit                   !> Unit testing framework
USE system_constants        !> Global constants
USE log_message_control     !> Printing log/error messages
USE log_messages            !> Log/error codes
USE field_data              !> Field Data module

IMPLICIT NONE


CONTAINS


! ------------------------------------------------------------------------
!>  \test Test for EXCEED exception on time-stepping initialization
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param dt         Time step size (within limits)
!!  \param n          Number of time steps (within limits)
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!
!!  This test makes sure that the EXCEED exception is raised when time
!!  stepping information is initialized to values outside defined limits.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_init_time_EXCEED
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_init_time_EXCEED'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  DOUBLE PRECISION, PARAMETER :: dt=1.d-3
  INTEGER, PARAMETER :: n=20
  INTEGER, PARAMETER :: expMsg=EXCEED
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log message file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> attempt to initialize with time step too small
  CALL fld_initTime(DTIME_MIN-dt,n, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> attempt to initialize with time step too large
  CALL fld_initTime(DTIME_MAX+dt,n, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> attempt to initialize with too few time steps
  CALL fld_initTime(dt,0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> attempt to initialize with too many time steps
  CALL fld_initTime(dt,MAX_TIMESTEPS+n, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize log file and reset time-stepping data
  CALL log_closeLogFile()
  CALL fld_cleanTime()

END SUBROUTINE test_fld_init_time_EXCEED


! ------------------------------------------------------------------------
!>  \test Test for time step when module is not initialized
!!
!!  \param unit_name    Name of unit test (for FRUIT)
!!  \param expected     Expected time step
!!  \param actual       Actual time step
!!
!!  This test makes sure that the time step is returned as 0 when the
!!  time-stepping data is not initialized
! ------------------------------------------------------------------------
SUBROUTINE test_fld_time_step_not_initialized
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_time_step_not_initialized'
  DOUBLE PRECISION, PARAMETER :: expected = 0.d0
  DOUBLE PRECISION :: actual

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> check size of time step
  actual = fld_timeStep()
  CALL assertEquals(expected, actual)

END SUBROUTINE test_fld_time_step_not_initialized


! ------------------------------------------------------------------------
!>  \test Test for time step when module is initialized
!!
!!  \param unit_name    Name of unit test (for FRUIT)
!!  \param dt           Time step
!!  \param ntime        Number of time steps
!!  \param expected     Expected time step
!!  \param actual       Actual time step
!!
!!  This test makes sure that the time step is returned correctly when the
!!  time-stepping data is initialized
! ------------------------------------------------------------------------
SUBROUTINE test_fld_time_step_initialized
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_time_step_initialized'
  DOUBLE PRECISION, PARAMETER :: dt = 1.d-3
  INTEGER, PARAMETER :: ntime = 100
  DOUBLE PRECISION, PARAMETER :: expected = dt
  DOUBLE PRECISION :: actual

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize time-stepping data
  CALL fld_initTime(dt,ntime)

  !> check size of time step
  actual = fld_timeStep()
  CALL assertEquals(expected, actual)

  !> reset time-stepping data
  CALL fld_cleanTime()

END SUBROUTINE test_fld_time_step_initialized


! ------------------------------------------------------------------------
!>  \test Test for number of time steps when module is not initialized
!!
!!  \param unit_name    Name of unit test (for FRUIT)
!!  \param expected     Expected number of time steps
!!  \param actual       Actual number of time steps
!!
!!  This test makes sure that the number of time steps is returned as 0
!!  when the time-stepping data is not initialized
! ------------------------------------------------------------------------
SUBROUTINE test_fld_num_time_step_not_initialized
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_num_time_step_not_initialized'
  INTEGER, PARAMETER :: expected = 0
  INTEGER :: actual

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> check number of time steps
  actual = fld_numTimeSteps()
  CALL assertEquals(expected, actual)

END SUBROUTINE test_fld_num_time_step_not_initialized


! ------------------------------------------------------------------------
!>  \test Test for time step when module is initialized
!!
!!  \param unit_name    Name of unit test (for FRUIT)
!!  \param dt           Time step
!!  \param ntime        Number of time steps
!!  \param expected     Expected number of time steps
!!  \param actual       Actual number of time steps
!!
!!  This test makes sure that the number of time steps is returned
!!  correctly when the time-stepping data is initialized
! ------------------------------------------------------------------------
SUBROUTINE test_fld_num_time_step_initialized
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_num_time_step_initialized'
  DOUBLE PRECISION, PARAMETER :: dt = 1.d-3
  INTEGER, PARAMETER :: ntime = 100
  INTEGER, PARAMETER :: expected = ntime
  INTEGER :: actual

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize time-stepping data
  CALL fld_initTime(dt,ntime)

  !> check size of time step
  actual = fld_numTimeSteps()
  CALL assertEquals(expected, actual)

  !> reset time-stepping data
  CALL fld_cleanTime()

END SUBROUTINE test_fld_num_time_step_initialized


! ------------------------------------------------------------------------
!>  \test Test for OK exception message on allocation of nodes
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!
!!  This test initializes the nodes state variable in the field_data
!!  module and makes sure that the exception message is OK
!!  (i.e. allocation did not fail).
! ------------------------------------------------------------------------
SUBROUTINE test_fld_node_allocation_MSG
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_node_allocation_MSG'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: expMsg=OK
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log message file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize module and check the exception
  CALL fld_initNode(nnod, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize log file and deallocate module
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_node_allocation_MSG


! ------------------------------------------------------------------------
!>  \test Test for SZE exception message on allocation of nodes
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!
!!  This test attempts to initialize the nodes state variable in the field
!!  data module with invalid size parameters and verifies that the correct
!!  exception is returned.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_node_allocation_SZE
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_node_allocation_SZE'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: expMsg=SZE
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> try to initialize with nnod=0
  CALL fld_initNode(0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> ensure module is reset
  CALL fld_cleanNode()

  !> try to initialize with nnod=MAX_NODES+1
  CALL fld_initNode(MAX_NODES+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize log file and deallocate module
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_node_allocation_SZE


! ------------------------------------------------------------------------
!>  \test Test for number of node when node data is not initialized
!!
!!  \param unit_name    Name of unit test (for FRUIT)
!!  \param expected     Expected number of nodes
!!  \param actual       Actual number of nodes
!!
!!  This test makes sure that the number of nodes is returned as 0
!!  when the node data is not initialized
! ------------------------------------------------------------------------
SUBROUTINE test_fld_num_node_not_allocated
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_num_node_not_allocated'
  INTEGER, PARAMETER :: expected = 0
  INTEGER :: actual

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> check number of materials
  actual = fld_numNode()
  CALL assertEquals(expected, actual)

END SUBROUTINE test_fld_num_node_not_allocated


! ------------------------------------------------------------------------
!>  \test Test for number of nodes when node data is initialized
!!
!!  \param unit_name    Name of unit test (for FRUIT)
!!  \param expected     Expected number of nodes
!!  \param actual       Actual number of nodes
!!
!!  This test makes sure that the correct number of nodes is returned.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_num_node_allocated
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_num_node_allocated'
  INTEGER, PARAMETER :: expected = 100
  INTEGER :: actual

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize the module
  CALL fld_initNode(expected)

  !> check number of nodes
  actual = fld_numNode()
  CALL assertEquals(expected, actual)

  !> deallocate the module
  CALL fld_cleanNode()

END SUBROUTINE test_fld_num_node_allocated


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_getCoord
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param x          Dummy variable for get function return
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested coordinate
!!  direction is not within [1..NDIM].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_coord_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_coord_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION :: x

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to get beyond last node
  x = fld_getCoord(nnod+1,jdim, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first node
  x = fld_getCoord(0,jdim, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get beyond last coord
  x = fld_getCoord(inod,NDIM+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first coord
  x = fld_getCoord(inod,0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_coord_POSIT


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_setCoord
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param x          Dummy variable for set routine input
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested coordinate
!!  direction is not within [1..NDIM].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_coord_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_coord_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION, PARAMETER :: x = 3.d0

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to set beyond last node
  CALL fld_setCoord(nnod+1,jdim, x, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first node
  CALL fld_setCoord(0,jdim, x, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set beyond last coord
  CALL fld_setCoord(inod,NDIM+1, x, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first coord
  CALL fld_setCoord(inod,0, x, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_set_coord_POSIT


! ------------------------------------------------------------------------
!>  \test Test for EXCEED exception from fld_setCoord
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!
!!  This test checks that an EXCEED exception is returned when the
!!  specified input is not within the range defined in the System
!!  Constants module.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_coord_EXCEED
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_coord_EXCEED'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = EXCEED
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to set below minimum
  CALL fld_setCoord(inod,jdim, COORD_MIN-0.1d0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set above maximum
  CALL fld_setCoord(inod,jdim, COORD_MAX+0.1d0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_set_coord_EXCEED


! ------------------------------------------------------------------------
!>  \test Test for correct value getting and setting in fld_getCoord and fld_setCoord
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expVal     Expected data value
!!  \param actVal     Actual data value
!!
!!  This test checks that the correct value is set using the fld_setCoord
!!  access program and returned from the fld_getCoord access program.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_set_coord_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_set_coord_VAL'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  DOUBLE PRECISION, PARAMETER :: expVal = 3.d0
  DOUBLE PRECISION :: actVal

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> set coord value
  CALL fld_setCoord(inod,jdim, expVal)
  actVal = fld_getCoord(inod,jdim)
  CALL assertEquals(expVal,actVal)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_set_coord_VAL


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_getFix
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param fix        Dummy variable for get function return
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested coordinate
!!  direction is not within [1..NDIM].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_fix_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_fix_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  LOGICAL :: fix

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to get beyond last node
  fix = fld_getFix(nnod+1,jdim, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first node
  fix = fld_getFix(0,jdim, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get beyond last coord
  fix = fld_getFix(inod,NDIM+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first coord
  fix = fld_getFix(inod,0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_fix_POSIT


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_setFix
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param fix        Dummy variable for set routine input
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested coordinate
!!  direction is not within [1..NDIM].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_fix_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_fix_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  LOGICAL, PARAMETER :: fix = .FALSE.

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to set beyond last node
  CALL fld_setFix(nnod+1,jdim, fix, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first node
  CALL fld_setFix(0,jdim, fix, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set beyond last coord
  CALL fld_setFix(inod,NDIM+1, fix, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first coord
  CALL fld_setFix(inod,0, fix, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_set_fix_POSIT


! ------------------------------------------------------------------------
!>  \test Test for correct value getting and setting in fld_getFix and fld_setFix
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expVal     Expected data value
!!  \param actVal     Actual data value
!!
!!  This test checks that the correct value is set using the fld_setCoord
!!  access program and returned from the fld_getCoord access program.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_set_fix_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_set_fix_VAL'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  LOGICAL, PARAMETER :: expVal = .TRUE.
  LOGICAL :: actVal

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> set coord value
  CALL fld_setFix(inod,jdim, expVal)
  actVal = fld_getFix(inod,jdim)
  CALL assertEquals(expVal,actVal)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_set_fix_VAL


! ------------------------------------------------------------------------
!>  \test Test for SZE exception message on initialization of degrees of freedom
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param expDof     Expected number of dofs (after failed init)
!!  \param actDof     Expected number of dofs
!!
!!  This test attempts to initialize the degree of freedom numbers in a
!!  situation where the number of dofs will exceed the prescribed limit.
!!  This is done by creating a system with the maximum number of nodes.
!!  The limit on the number of degrees of freedom is set such that it is
!!  less than what would be allowed by a system with no fixities and the
!!  maximum number of nodes.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_dof_initialization_SZE
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_dof_initialization_SZE'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod = MAX_NODES
  INTEGER, PARAMETER :: expMsg=SZE
  INTEGER, PARAMETER :: expDof=0
  INTEGER :: actMsg, actDof

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize node data
  CALL fld_initNode(nnod)

  !> attempt to initialize degrees of freedom
  CALL fld_initDof(actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> check the number of dofs
  actDof = fld_numDof()
  CALL assertEquals(expDof,actDof)

  !> finalize log file and deallocate module
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_dof_initialization_SZE


! ------------------------------------------------------------------------
!>  \test Test for correct degree of freedom count when dofs not initialized
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param nnod       Number of nodes
!!  \param expDof     Expected number of dofs
!!  \param actDof     Expected number of dofs
!!
!!  This test makes sure that the number of degrees of freedom is returned
!!  as 0 when the dofs are not initialized.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_num_dof_not_initialized
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_num_dof_not_initialized'
  INTEGER, PARAMETER :: nnod = 100
  INTEGER, PARAMETER :: expDof=0
  INTEGER :: actDof

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize node data
  CALL fld_initNode(nnod)

  !> check the number of dofs
  actDof = fld_numDof()
  CALL assertEquals(expDof,actDof)

  !> deallocate nodes
  CALL fld_cleanNode()

END SUBROUTINE test_fld_num_dof_not_initialized


! ------------------------------------------------------------------------
!>  \test Test for correct degree of freedom count when dofs are initialized
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param nnod       Number of nodes
!!  \param ifix1      Node number for first fixity
!!  \param jfix1      Coordinate index for first fixity
!!  \param ifix2      Node number for second fixity
!!  \param jfix2      Coordinate index for second fixity
!!  \param expDof     Expected number of dofs
!!  \param actDof     Expected number of dofs
!!
!!  This test initializes the degree of freedom numbers and makes sure
!!  that the correct number of dofs is calculated.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_num_dof_initialized
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_num_dof_initialized'
  INTEGER, PARAMETER :: nnod = 100
  INTEGER, PARAMETER :: ifix1=5,jfix1=2, ifix2=23,jfix2=1
  INTEGER, PARAMETER :: expDof=NNOD*NDIM-2
  INTEGER :: actDof

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize node data
  CALL fld_initNode(nnod)

  !> set fixities
  CALL fld_setFix(ifix1,jfix1, .TRUE.)
  CALL fld_setFix(ifix2,jfix2, .TRUE.)

  !> initialize degrees of freedom
  CALL fld_initDof()

  !> check the number of dofs
  actDof = fld_numDof()
  CALL assertEquals(expDof,actDof)

  !> deallocate nodes
  CALL fld_cleanNode()

END SUBROUTINE test_fld_num_dof_initialized


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_getDof
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param dof        Dummy variable for get function return
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested coordinate
!!  direction is not within [1..NDIM].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_dof_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_dof_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  INTEGER :: dof

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to get beyond last node
  dof = fld_getDof(nnod+1,jdim, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first node
  dof = fld_getDof(0,jdim, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get beyond last coord
  dof = fld_getDof(inod,NDIM+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first coord
  dof = fld_getDof(inod,0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_dof_POSIT


! ------------------------------------------------------------------------
!>  \test Test for correct degree of freedom value return
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param nnod       Number of nodes
!!  \param ifix1      Node number for first fixity
!!  \param jfix1      Coordinate index for first fixity
!!  \param ifix2      Node number for second fixity
!!  \param jfix2      Coordinate index for second fixity
!!  \param idof1      Node number for first dof check
!!  \param jdof1      Coordinate index for first dof check
!!  \param expDof1    Expected dof number for first dof check
!!  \param actDof1    Actual dof number for first dof check
!!  \param idof2      Node number for second dof check
!!  \param jdof2      Coordinate index for second dof check
!!  \param expDof2    Expected dof number for second dof check
!!  \param actDof2    Actual dof number for second dof check
!!
!!  This test initializes the degree of freedom numbers and checks that
!!  specific degrees of freedom are returned as expected.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_dof_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_dof_VAL'
  INTEGER, PARAMETER :: nnod = 100
  INTEGER, PARAMETER :: ifix1=5,jfix1=2, ifix2=23,jfix2=1
  INTEGER, PARAMETER :: idof1=ifix1, jdof1=jfix1
  INTEGER, PARAMETER :: idof2=ifix2+1, jdof2=jfix2
  INTEGER, PARAMETER :: expDof1 = 0
  INTEGER, PARAMETER :: expDof2 = (idof2-1)*NDIM + jdof2 - 2
  INTEGER :: actDof1, actDof2

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize node data
  CALL fld_initNode(nnod)

  !> set fixities
  CALL fld_setFix(ifix1,jfix1, .TRUE.)
  CALL fld_setFix(ifix2,jfix2, .TRUE.)

  !> initialize degrees of freedom
  CALL fld_initDof()

  !> check the dof numbers
  actDof1 = fld_getDof(idof1,jdof1)
  CALL assertEquals(expDof1,actDof1)

  actDof2 = fld_getDof(idof2,jdof2)
  CALL assertEquals(expDof2,actDof2)

  !> deallocate nodes
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_dof_VAL


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_getDisp
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param u          Dummy variable for get function return
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested coordinate
!!  direction is not within [1..NDIM].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_disp_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_disp_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION :: u

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to get beyond last node
  u = fld_getDisp(nnod+1,jdim, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first node
  u = fld_getDisp(0,jdim, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get beyond last coord
  u = fld_getDisp(inod,NDIM+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first coord
  u = fld_getDisp(inod,0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_disp_POSIT


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_setDisp
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param u          Dummy variable for set routine input
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested coordinate
!!  direction is not within [1..NDIM].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_disp_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_disp_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION, PARAMETER :: u = 3.d0

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to set beyond last node
  CALL fld_setDisp(nnod+1,jdim, u, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first node
  CALL fld_setDisp(0,jdim, u, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set beyond last coord
  CALL fld_setDisp(inod,NDIM+1, u, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first coord
  CALL fld_setDisp(inod,0, u, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_set_disp_POSIT


! ------------------------------------------------------------------------
!>  \test Test for EXCEED exception from fld_setDisp
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!
!!  This test checks that an EXCEED exception is returned when the
!!  specified input is not within the range defined in the System
!!  Constants module.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_disp_EXCEED
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_disp_EXCEED'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = EXCEED
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to set below minimum
  CALL fld_setDisp(inod,jdim, DISP_MIN-0.1d0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set above maximum
  CALL fld_setDisp(inod,jdim, DISP_MAX+0.1d0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_set_disp_EXCEED


! ------------------------------------------------------------------------
!>  \test Test for correct value getting and setting in fld_getDisp and fld_setDisp
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expVal     Expected data value
!!  \param actVal     Actual data value
!!
!!  This test checks that the correct value is set using the fld_setDisp
!!  access program and returned from the fld_getDisp access program.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_set_disp_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_set_disp_VAL'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  DOUBLE PRECISION, PARAMETER :: expVal = 3.d0
  DOUBLE PRECISION :: actVal

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> set coord value
  CALL fld_setDisp(inod,jdim, expVal)
  actVal = fld_getDisp(inod,jdim)
  CALL assertEquals(expVal,actVal)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_set_disp_VAL


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_getVel
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param v          Dummy variable for get function return
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested coordinate
!!  direction is not within [1..NDIM].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_vel_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_vel_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION :: v

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to get beyond last node
  v = fld_getVel(nnod+1,jdim, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first node
  v = fld_getVel(0,jdim, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get beyond last coord
  v = fld_getVel(inod,NDIM+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first coord
  v = fld_getVel(inod,0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_vel_POSIT


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_setVel
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param v          Dummy variable for set routine input
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested coordinate
!!  direction is not within [1..NDIM].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_vel_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_vel_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION, PARAMETER :: v = 3.d0

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to set beyond last node
  CALL fld_setVel(nnod+1,jdim, v, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first node
  CALL fld_setVel(0,jdim, v, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set beyond last coord
  CALL fld_setVel(inod,NDIM+1, v, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first coord
  CALL fld_setVel(inod,0, v, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_set_vel_POSIT


! ------------------------------------------------------------------------
!>  \test Test for EXCEED exception from fld_setVel
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!
!!  This test checks that an EXCEED exception is returned when the
!!  specified input is not within the range defined in the System
!!  Constants module.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_vel_EXCEED
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_vel_EXCEED'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = EXCEED
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to set below minimum
  CALL fld_setVel(inod,jdim, VEL_MIN-0.1d0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set above maximum
  CALL fld_setVel(inod,jdim, VEL_MAX+0.1d0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_set_vel_EXCEED


! ------------------------------------------------------------------------
!>  \test Test for correct value getting and setting in fld_getVel and fld_setVel
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expVal     Expected data value
!!  \param actVal     Actual data value
!!
!!  This test checks that the correct value is set using the fld_setVel
!!  access program and returned from the fld_getVel access program.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_set_vel_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_set_vel_VAL'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  DOUBLE PRECISION, PARAMETER :: expVal = 3.d0
  DOUBLE PRECISION :: actVal

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> set coord value
  CALL fld_setVel(inod,jdim, expVal)
  actVal = fld_getVel(inod,jdim)
  CALL assertEquals(expVal,actVal)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_set_vel_VAL


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_getAcc
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param a          Dummy variable for get function return
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested coordinate
!!  direction is not within [1..NDIM].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_acc_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_acc_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION :: a

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to get beyond last node
  a = fld_getAcc(nnod+1,jdim, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first node
  a = fld_getAcc(0,jdim, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get beyond last coord
  a = fld_getAcc(inod,NDIM+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first coord
  a = fld_getAcc(inod,0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_acc_POSIT


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_setAcc
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param a          Dummy variable for set routine input
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested coordinate
!!  direction is not within [1..NDIM].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_acc_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_acc_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION, PARAMETER :: a = 3.d0

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to set beyond last node
  CALL fld_setAcc(nnod+1,jdim, a, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first node
  CALL fld_setAcc(0,jdim, a, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set beyond last coord
  CALL fld_setAcc(inod,NDIM+1, a, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first coord
  CALL fld_setAcc(inod,0, a, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_set_acc_POSIT


! ------------------------------------------------------------------------
!>  \test Test for EXCEED exception from fld_setAcc
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!
!!  This test checks that an EXCEED exception is returned when the
!!  specified input is not within the range defined in the System
!!  Constants module.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_acc_EXCEED
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_acc_EXCEED'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = EXCEED
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to set below minimum
  CALL fld_setAcc(inod,jdim, ACC_MIN-0.1d0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set above maximum
  CALL fld_setAcc(inod,jdim, ACC_MAX+0.1d0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_set_acc_EXCEED


! ------------------------------------------------------------------------
!>  \test Test for correct value getting and setting in fld_getAcc and fld_setAcc
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expVal     Expected data value
!!  \param actVal     Actual data value
!!
!!  This test checks that the correct value is set using the fld_setAcc
!!  access program and returned from the fld_getAcc access program.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_set_acc_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_set_acc_VAL'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  DOUBLE PRECISION, PARAMETER :: expVal = 3.d0
  DOUBLE PRECISION :: actVal

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> set coord value
  CALL fld_setAcc(inod,jdim, expVal)
  actVal = fld_getAcc(inod,jdim)
  CALL assertEquals(expVal,actVal)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_set_acc_VAL


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_getBodyAcc
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param a          Dummy variable for get function return
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested coordinate
!!  direction is not within [1..NDIM].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_body_acc_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_body_acc_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION :: a

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to get beyond last node
  a = fld_getBodyAcc(nnod+1,jdim, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first node
  a = fld_getBodyAcc(0,jdim, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get beyond last coord
  a = fld_getBodyAcc(inod,NDIM+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first coord
  a = fld_getBodyAcc(inod,0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_body_acc_POSIT


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_setBodyAcc
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param a          Dummy variable for set routine input
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested coordinate
!!  direction is not within [1..NDIM].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_body_acc_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_body_acc_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION, PARAMETER :: a = 3.d0

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to set beyond last node
  CALL fld_setBodyAcc(nnod+1,jdim, a, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first node
  CALL fld_setBodyAcc(0,jdim, a, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set beyond last coord
  CALL fld_setBodyAcc(inod,NDIM+1, a, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first coord
  CALL fld_setBodyAcc(inod,0, a, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_set_body_acc_POSIT


! ------------------------------------------------------------------------
!>  \test Test for EXCEED exception from fld_setBodyAcc
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!
!!  This test checks that an EXCEED exception is returned when the
!!  specified input is not within the range defined in the System
!!  Constants module.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_body_acc_EXCEED
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_body_acc_EXCEED'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  INTEGER, PARAMETER :: expMsg = EXCEED
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to set below minimum
  CALL fld_setBodyAcc(inod,jdim, ACC_MIN-0.1d0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set above maximum
  CALL fld_setBodyAcc(inod,jdim, ACC_MAX+0.1d0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_set_body_acc_EXCEED


! ------------------------------------------------------------------------
!>  \test Test for correct value getting and setting in fld_getBodyAcc and fld_setBodyAcc
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jdim       Coordinate index
!!  \param expVal     Expected data value
!!  \param actVal     Actual data value
!!
!!  This test checks that the correct value is set using the fld_setBodyAcc
!!  access program and returned from the fld_getBodyAcc access program.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_set_body_acc_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_set_body_acc_VAL'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jdim = 2
  DOUBLE PRECISION, PARAMETER :: expVal = 3.d0
  DOUBLE PRECISION :: actVal

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> set coord value
  CALL fld_setBodyAcc(inod,jdim, expVal)
  actVal = fld_getBodyAcc(inod,jdim)
  CALL assertEquals(expVal,actVal)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_set_body_acc_VAL


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_getStressNode
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jtns       Tensor index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param s          Dummy variable for get function return
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested tensor index
!!  is not within [1..NTNS].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_stress_node_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_stress_node_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jtns = s22
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION :: s

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to get beyond last node
  s = fld_getStressNode(nnod+1,jtns, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first node
  s = fld_getStressNode(0,jtns, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get beyond last tensor index
  s = fld_getStressNode(inod,NTNS+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first tensor index
  s = fld_getStressNode(inod,0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_stress_node_POSIT


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_setStressNode
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jtns       Tensor index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param s          Dummy variable for set routine input
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested tensor index
!!  is not within [1..NTNS].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_stress_node_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_stress_node_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jtns = s22
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION, PARAMETER :: s = 3.d0

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to set beyond last node
  CALL fld_setStressNode(nnod+1,jtns, s, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first node
  CALL fld_setStressNode(0,jtns, s, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set beyond last tensor index
  CALL fld_setStressNode(inod,NTNS+1, s, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first tensor index
  CALL fld_setStressNode(inod,0, s, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_set_stress_node_POSIT


! ------------------------------------------------------------------------
!>  \test Test for EXCEED exception from fld_setStressNode
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jtns       Tensor index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!
!!  This test checks that an EXCEED exception is returned when the
!!  specified input is not within the range defined in the System
!!  Constants module.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_stress_node_EXCEED
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_stress_node_EXCEED'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jtns = s22
  INTEGER, PARAMETER :: expMsg = EXCEED
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to set below minimum
  CALL fld_setStressNode(inod,jtns, SIG_MIN-0.1d0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set above maximum
  CALL fld_setStressNode(inod,jtns, SIG_MAX+0.1d0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_set_stress_node_EXCEED


! ------------------------------------------------------------------------
!>  \test Test for correct value getting and setting in fld_getStressNode
!!        and fld_setStressNode
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jtns       Tensor index
!!  \param expVal     Expected data value
!!  \param actVal     Actual data value
!!
!!  This test checks that the correct value is set using the
!!  fld_setStressNode access program and returned from the
!!  fld_getStressNode access program.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_set_stress_node_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_set_stress_node_VAL'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jtns = s22
  DOUBLE PRECISION, PARAMETER :: expVal = 3.d0
  DOUBLE PRECISION :: actVal

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> set coord value
  CALL fld_setStressNode(inod,jtns, expVal)
  actVal = fld_getStressNode(inod,jtns)
  CALL assertEquals(expVal,actVal)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_set_stress_node_VAL


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_getStrainNode
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jtns       Tensor index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param s          Dummy variable for get function return
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested tensor index
!!  is not within [1..NTNS].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_strain_node_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_strain_node_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jtns = s22
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION :: s

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to get beyond last node
  s = fld_getStrainNode(nnod+1,jtns, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first node
  s = fld_getStrainNode(0,jtns, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get beyond last tensor index
  s = fld_getStrainNode(inod,NTNS+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first tensor index
  s = fld_getStrainNode(inod,0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_strain_node_POSIT


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_setStrainNode
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jtns       Tensor index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param s          Dummy variable for set routine input
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested tensor index
!!  is not within [1..NTNS].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_strain_node_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_strain_node_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jtns = s22
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION, PARAMETER :: s = 3.d-3

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to set beyond last node
  CALL fld_setStrainNode(nnod+1,jtns, s, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first node
  CALL fld_setStrainNode(0,jtns, s, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set beyond last tensor index
  CALL fld_setStrainNode(inod,NTNS+1, s, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first tensor index
  CALL fld_setStrainNode(inod,0, s, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_set_strain_node_POSIT


! ------------------------------------------------------------------------
!>  \test Test for EXCEED exception from fld_setStrainNode
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jtns       Tensor index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!
!!  This test checks that an EXCEED exception is returned when the
!!  specified input is not within the range defined in the System
!!  Constants module.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_strain_node_EXCEED
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_strain_node_EXCEED'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jtns = s22
  INTEGER, PARAMETER :: expMsg = EXCEED
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> try to set below minimum
  CALL fld_setStrainNode(inod,jtns, STR_MIN-0.1d-4, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set above maximum
  CALL fld_setStrainNode(inod,jtns, STR_MAX+0.1d-4, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_set_strain_node_EXCEED


! ------------------------------------------------------------------------
!>  \test Test for correct value getting and setting in fld_getStrainNode
!!        and fld_setStrainNode
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param nnod       Number of nodes
!!  \param inod       Node number
!!  \param jtns       Tensor index
!!  \param expVal     Expected data value
!!  \param actVal     Actual data value
!!
!!  This test checks that the correct value is set using the
!!  fld_setStrainNode access program and returned from the
!!  fld_getStrainNode access program.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_set_strain_node_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_set_strain_node_VAL'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: inod = 50
  INTEGER, PARAMETER :: jtns = s22
  DOUBLE PRECISION, PARAMETER :: expVal = 3.d-3
  DOUBLE PRECISION :: actVal

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize nodes
  CALL fld_initNode(nnod)

  !> set coord value
  CALL fld_setStrainNode(inod,jtns, expVal)
  actVal = fld_getStrainNode(inod,jtns)
  CALL assertEquals(expVal,actVal)

  !> finalize the log file and deallocate the nodes list
  CALL log_closeLogFile()
  CALL fld_cleanNode()

END SUBROUTINE test_fld_get_set_strain_node_VAL


! ------------------------------------------------------------------------
!>  \test Test for OK exception message on allocation of elements
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nel        Number of elements
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!
!!  This test initializes the elements state variable in the field_data
!!  module and makes sure that the exception message is OK
!!  (i.e. allocation did not fail).
! ------------------------------------------------------------------------
SUBROUTINE test_fld_elem_allocation_MSG
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_elem_allocation_MSG'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nel=100
  INTEGER, PARAMETER :: expMsg=OK
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log message file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize module and check the exception
  CALL fld_initElem(nel, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize log file and deallocate module
  CALL log_closeLogFile()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_elem_allocation_MSG


! ------------------------------------------------------------------------
!>  \test Test for SZE exception message on allocation of elements
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!
!!  This test attempts to initialize the elements state variable in the
!!  field data module with invalid size parameters and verifies that the
!!  correct exception is returned.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_elem_allocation_SZE
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_elem_allocation_SZE'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: expMsg=SZE
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> try to initialize with nel=0
  CALL fld_initElem(0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> ensure module is reset
  CALL fld_cleanElem()

  !> try to initialize with nel=MAX_ELEMENTS+1
  CALL fld_initElem(MAX_ELEMENTS+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize log file and deallocate module
  CALL log_closeLogFile()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_elem_allocation_SZE


! ------------------------------------------------------------------------
!>  \test Test for number of elements when element data is not initialized
!!
!!  \param unit_name    Name of unit test (for FRUIT)
!!  \param expected     Expected number of elements
!!  \param actual       Actual number of elements
!!
!!  This test makes sure that the number of elements is returned as 0
!!  when the element data is not initialized
! ------------------------------------------------------------------------
SUBROUTINE test_fld_num_elem_not_allocated
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_num_elem_not_allocated'
  INTEGER, PARAMETER :: expected = 0
  INTEGER :: actual

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> check number of materials
  actual = fld_numElem()
  CALL assertEquals(expected, actual)

END SUBROUTINE test_fld_num_elem_not_allocated


! ------------------------------------------------------------------------
!>  \test Test for number of elements when element data is initialized
!!
!!  \param unit_name    Name of unit test (for FRUIT)
!!  \param expected     Expected number of elements
!!  \param actual       Actual number of elements
!!
!!  This test makes sure that the correct number of elements is returned.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_num_elem_allocated
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_num_elem_allocated'
  INTEGER, PARAMETER :: expected = 100
  INTEGER :: actual

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize the module
  CALL fld_initElem(expected)

  !> check number of nodes
  actual = fld_numElem()
  CALL assertEquals(expected, actual)

  !> deallocate the module
  CALL fld_cleanElem()

END SUBROUTINE test_fld_num_elem_allocated


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_getConnect
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nel        Number of elements
!!  \param iel        Element number
!!  \param jnod       Local node index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param n          Dummy variable for get function return
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the element list or the requested local node
!!  index is not within [1..NNODEL].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_connect_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_connect_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nel=100
  INTEGER, PARAMETER :: iel = 50
  INTEGER, PARAMETER :: jnod = 2
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  INTEGER :: n

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes
  CALL fld_initElem(nel)

  !> try to get beyond last element
  n = fld_getConnect(nel+1,jnod, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first element
  n = fld_getConnect(0,jnod, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get beyond last local node index
  n = fld_getConnect(iel,NNODEL+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first local node index
  n = fld_getConnect(iel,0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the elements list
  CALL log_closeLogFile()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_get_connect_POSIT


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_setConnect
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param nel        Number of elements
!!  \param iel        Element number
!!  \param jnod       Local node index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param n          Dummy variable for set routine input
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the element list or the requested local node
!!  index is not within [1..NNODEL].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_connect_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_connect_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: nel=100
  INTEGER, PARAMETER :: iel = 50
  INTEGER, PARAMETER :: jnod = 2
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  INTEGER, PARAMETER :: n = 10

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes and elements
  CALL fld_initNode(nnod)
  CALL fld_initElem(nel)

  !> try to set beyond last element
  CALL fld_setConnect(nel+1,jnod, n, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first element
  CALL fld_setConnect(0,jnod, n, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set beyond last local node index
  CALL fld_setConnect(iel,NNODEL+1, n, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first local node index
  CALL fld_setConnect(iel,0, n, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes and elements
  CALL log_closeLogFile()
  CALL fld_cleanNode()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_set_connect_POSIT


! ------------------------------------------------------------------------
!>  \test Test for EXCEED exception from fld_setConnect
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nnod       Number of nodes
!!  \param nel        Number of elements
!!  \param iel        Element number
!!  \param jnod       Local node index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!
!!  This test checks that an EXCEED exception is returned when the
!!  specified input is not within the range defined in the System
!!  Constants module.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_connect_EXCEED
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_connect_EXCEED'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: nel=100
  INTEGER, PARAMETER :: iel = 50
  INTEGER, PARAMETER :: jnod = 2
  INTEGER, PARAMETER :: expMsg = EXCEED
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize nodes and elements
  CALL fld_initNode(nnod)
  CALL fld_initElem(nel)

  !> try to set below minimum
  CALL fld_setConnect(iel,jnod, 0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set above maximum
  CALL fld_setConnect(iel,jnod, nnod+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the nodes and elements
  CALL log_closeLogFile()
  CALL fld_cleanNode()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_set_connect_EXCEED


! ------------------------------------------------------------------------
!>  \test Test for correct value getting and setting in fld_getConnect
!!        and fld_setConnect
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param nnod       Number of nodes
!!  \param nel        Number of elements
!!  \param iel        Element number
!!  \param jnod       Local node index
!!  \param expVal     Expected data value
!!  \param actVal     Actual data value
!!
!!  This test checks that the correct value is set using the
!!  fld_setConnect access program and returned from the
!!  fld_getConnect access program.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_set_connect_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_set_connect_VAL'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: nel=100
  INTEGER, PARAMETER :: iel = 50
  INTEGER, PARAMETER :: jnod = 2
  INTEGER, PARAMETER :: expVal = 20
  INTEGER :: actVal

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize nodes and elements
  CALL fld_initNode(nnod)
  CALL fld_initElem(nel)

  !> set connectivity value
  CALL fld_setConnect(iel,jnod, expVal)
  actVal = fld_getConnect(iel,jnod)
  CALL assertEquals(expVal,actVal)

  !> finalize the log file and deallocate the nodes and elements
  CALL log_closeLogFile()
  CALL fld_cleanNode()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_get_set_connect_VAL


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception message from fld_volElem
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nel        Number of elements
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param vol        Dummy variable for get function return
!!
!!  This test makes sure that the correct exception is returned when the
!!  specified element index is not in the range of the allocated element
!!  data.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_vol_elem_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_vol_elem_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nel=100
  INTEGER, PARAMETER :: expMsg=POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION :: vol

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize elements
  CALL fld_initElem(nel)

  !> try to get beyond last element
  vol = fld_volElem(nel+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first element
  vol = fld_volElem(0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the elements list
  CALL log_closeLogFile()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_vol_elem_POSIT


! ------------------------------------------------------------------------
!>  \test Test for element volume when node coordinates not initialized
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param nnod       Number of nodes
!!  \param nel        Number of elements
!!  \param iel        Element number
!!  \param expected   Expected volume of element (zero)
!!  \param actual     Actual volume of element
!!
!!  This test makes sure that the volume of the element is returned as 0
!!  when the coordinates of the connected nodes have not been set
! ------------------------------------------------------------------------
SUBROUTINE test_fld_vol_elem_ZERO
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_vol_elem_ZERO'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: nel=100
  DOUBLE PRECISION, PARAMETER :: expected = 0.d0
  DOUBLE PRECISION :: actual

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize nodes and elements
  CALL fld_initNode(nnod)
  CALL fld_initElem(nel)

  !> set connectivity of first element to {1,2,3}
  CALL fld_setConnect(1,1,1)
  CALL fld_setConnect(1,2,2)
  CALL fld_setConnect(1,3,3)

  !> check volume of element
  actual = fld_volElem(1)
  CALL assertEquals(expected,actual)

  !> deallocate the elements and nodes
  CALL fld_cleanNode()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_vol_elem_ZERO


! ------------------------------------------------------------------------
!>  \test Test for element volume when node coordinates initialized
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param nnod       Number of nodes
!!  \param nel        Number of elements
!!  \param iel        Element number
!!  \param expected   Expected volume of element
!!  \param actual     Actual volume of element
!!
!!  This test makes sure that the volume of the element is computed
!!  correctly for the triangle given by (1,1), (3,2), (2,3)
! ------------------------------------------------------------------------
SUBROUTINE test_fld_vol_elem_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_vol_elem_VAL'
  INTEGER, PARAMETER :: nnod=100
  INTEGER, PARAMETER :: nel=100
  DOUBLE PRECISION, PARAMETER :: toler = 1.d-14
  DOUBLE PRECISION, PARAMETER :: expected = 1.5d0
  DOUBLE PRECISION :: actual

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize nodes and elements
  CALL fld_initNode(nnod)
  CALL fld_initElem(nel)

  !> set connectivity of first element to {1,2,3}
  CALL fld_setConnect(1,1,1)
  CALL fld_setConnect(1,2,2)
  CALL fld_setConnect(1,3,3)

  !> set coordinates of nodes
  CALL fld_setCoord(1,1, 1.d0)
  CALL fld_setCoord(1,2, 1.d0)
  CALL fld_setCoord(2,1, 3.d0)
  CALL fld_setCoord(2,2, 2.d0)
  CALL fld_setCoord(3,1, 2.d0)
  CALL fld_setCoord(3,2, 3.d0)

  !> check volume of element
  actual = fld_volElem(1)
  CALL assertEquals(expected,actual, toler)

  !> deallocate the elements and nodes
  CALL fld_cleanNode()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_vol_elem_VAL


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_getStressElem
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nel        Number of elements
!!  \param iel        Element number
!!  \param jtns       Tensor index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param s          Dummy variable for get function return
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the element list or the requested tensor index
!!  is not within [1..NTNS].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_stress_elem_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_stress_elem_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nel=100
  INTEGER, PARAMETER :: iel = 50
  INTEGER, PARAMETER :: jtns = s22
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION :: s

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize elements
  CALL fld_initElem(nel)

  !> try to get beyond last element
  s = fld_getStressElem(nel+1,jtns, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first element
  s = fld_getStressElem(0,jtns, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get beyond last tensor index
  s = fld_getStressElem(iel,NTNS+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first tensor index
  s = fld_getStressElem(iel,0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the elements list
  CALL log_closeLogFile()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_get_stress_elem_POSIT


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_setStressElem
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nel        Number of elements
!!  \param iel        Element number
!!  \param jtns       Tensor index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param s          Dummy variable for set routine input
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the element list or the requested tensor index
!!  is not within [1..NTNS].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_stress_elem_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_stress_elem_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nel=100
  INTEGER, PARAMETER :: iel = 50
  INTEGER, PARAMETER :: jtns = s22
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION, PARAMETER :: s = 3.d0

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize elements
  CALL fld_initElem(nel)

  !> try to set beyond last element
  CALL fld_setStressElem(nel+1,jtns, s, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first element
  CALL fld_setStressElem(0,jtns, s, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set beyond last tensor index
  CALL fld_setStressElem(iel,NTNS+1, s, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first tensor index
  CALL fld_setStressElem(iel,0, s, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the elements list
  CALL log_closeLogFile()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_set_stress_elem_POSIT


! ------------------------------------------------------------------------
!>  \test Test for EXCEED exception from fld_setStressElem
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nel       Number of elements
!!  \param iel       Element number
!!  \param jtns       Tensor index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!
!!  This test checks that an EXCEED exception is returned when the
!!  specified input is not within the range defined in the System
!!  Constants module.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_stress_elem_EXCEED
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_stress_elem_EXCEED'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nel=100
  INTEGER, PARAMETER :: iel = 50
  INTEGER, PARAMETER :: jtns = s22
  INTEGER, PARAMETER :: expMsg = EXCEED
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize elements
  CALL fld_initElem(nel)

  !> try to set below minimum
  CALL fld_setStressElem(iel,jtns, SIG_MIN-0.1d0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set above maximum
  CALL fld_setStressElem(iel,jtns, SIG_MAX+0.1d0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the elements list
  CALL log_closeLogFile()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_set_stress_elem_EXCEED


! ------------------------------------------------------------------------
!>  \test Test for correct value getting and setting in fld_getStressElem
!!        and fld_setStressElem
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param nel       Number of elements
!!  \param iel       Element number
!!  \param jtns       Tensor index
!!  \param expVal     Expected data value
!!  \param actVal     Actual data value
!!
!!  This test checks that the correct value is set using the
!!  fld_setStressElem access program and returned from the
!!  fld_getStressElem access program.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_set_stress_elem_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_set_stress_elem_VAL'
  INTEGER, PARAMETER :: nel=100
  INTEGER, PARAMETER :: iel = 50
  INTEGER, PARAMETER :: jtns = s22
  DOUBLE PRECISION, PARAMETER :: expVal = 3.d0
  DOUBLE PRECISION :: actVal

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize elements
  CALL fld_initElem(nel)

  !> set coord value
  CALL fld_setStressElem(iel,jtns, expVal)
  actVal = fld_getStressElem(iel,jtns)
  CALL assertEquals(expVal,actVal)

  !> finalize the log file and deallocate the elements list
  CALL log_closeLogFile()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_get_set_stress_elem_VAL


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_getStrainElem
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nel       Number of elements
!!  \param iel       Element number
!!  \param jtns       Tensor index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param s          Dummy variable for get function return
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested tensor index
!!  is not within [1..NTNS].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_strain_elem_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_strain_elem_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nel=100
  INTEGER, PARAMETER :: iel = 50
  INTEGER, PARAMETER :: jtns = s22
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION :: s

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize elements
  CALL fld_initElem(nel)

  !> try to get beyond last node
  s = fld_getStrainElem(nel+1,jtns, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first node
  s = fld_getStrainElem(0,jtns, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get beyond last tensor index
  s = fld_getStrainElem(iel,NTNS+1, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to get before first tensor index
  s = fld_getStrainElem(iel,0, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the elements list
  CALL log_closeLogFile()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_get_strain_elem_POSIT


! ------------------------------------------------------------------------
!>  \test Test for POSIT exception from fld_setStrainElem
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nel       Number of elements
!!  \param iel       Element number
!!  \param jtns       Tensor index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!  \param s          Dummy variable for set routine input
!!
!!  This test checks that a POSIT exception is returned when the requested
!!  location is not inside the node list or the requested tensor index
!!  is not within [1..NTNS].
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_strain_elem_POSIT
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_strain_elem_POSIT'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nel=100
  INTEGER, PARAMETER :: iel = 50
  INTEGER, PARAMETER :: jtns = s22
  INTEGER, PARAMETER :: expMsg = POSIT
  INTEGER :: actMsg
  DOUBLE PRECISION, PARAMETER :: s = 3.d-3

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize elements
  CALL fld_initElem(nel)

  !> try to set beyond last node
  CALL fld_setStrainElem(nel+1,jtns, s, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first node
  CALL fld_setStrainElem(0,jtns, s, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set beyond last tensor index
  CALL fld_setStrainElem(iel,NTNS+1, s, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set before first tensor index
  CALL fld_setStrainElem(iel,0, s, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the elements list
  CALL log_closeLogFile()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_set_strain_elem_POSIT


! ------------------------------------------------------------------------
!>  \test Test for EXCEED exception from fld_setStrainElem
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param testName   Filename for log file (required for exceptions)
!!  \param nel       Number of elements
!!  \param iel       Element number
!!  \param jtns       Tensor index
!!  \param expMsg     Expected log message
!!  \param actMsg     Actual log message
!!
!!  This test checks that an EXCEED exception is returned when the
!!  specified input is not within the range defined in the System
!!  Constants module.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_set_strain_elem_EXCEED
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_set_strain_elem_EXCEED'
  CHARACTER (LEN=*), PARAMETER :: testName = 'testName'
  INTEGER, PARAMETER :: nel=100
  INTEGER, PARAMETER :: iel = 50
  INTEGER, PARAMETER :: jtns = s22
  INTEGER, PARAMETER :: expMsg = EXCEED
  INTEGER :: actMsg

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize log file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize elements
  CALL fld_initElem(nel)

  !> try to set below minimum
  CALL fld_setStrainElem(iel,jtns, STR_MIN-0.1d-4, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> try to set above maximum
  CALL fld_setStrainElem(iel,jtns, STR_MAX+0.1d-4, actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> finalize the log file and deallocate the elements list
  CALL log_closeLogFile()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_set_strain_elem_EXCEED


! ------------------------------------------------------------------------
!>  \test Test for correct value getting and setting in fld_getStrainElem
!!        and fld_setStrainElem
!!
!!  \param unit_name  Name of unit test (for FRUIT)
!!  \param nel       Number of elements
!!  \param iel       Element number
!!  \param jtns       Tensor index
!!  \param expVal     Expected data value
!!  \param actVal     Actual data value
!!
!!  This test checks that the correct value is set using the
!!  fld_setStrainElem access program and returned from the
!!  fld_getStrainElem access program.
! ------------------------------------------------------------------------
SUBROUTINE test_fld_get_set_strain_elem_VAL
  CHARACTER (LEN=*), PARAMETER :: unit_name = 'test_fld_get_set_strain_elem_VAL'
  INTEGER, PARAMETER :: nel=100
  INTEGER, PARAMETER :: iel = 50
  INTEGER, PARAMETER :: jtns = s22
  DOUBLE PRECISION, PARAMETER :: expVal = 3.d-3
  DOUBLE PRECISION :: actVal

  !> initialize unit test
  CALL set_unit_name(unit_name)

  !> initialize elements
  CALL fld_initElem(nel)

  !> set coord value
  CALL fld_setStrainElem(iel,jtns, expVal)
  actVal = fld_getStrainElem(iel,jtns)
  CALL assertEquals(expVal,actVal)

  !> finalize the log file and deallocate the elements list
  CALL log_closeLogFile()
  CALL fld_cleanElem()

END SUBROUTINE test_fld_get_set_strain_elem_VAL

END MODULE field_data_test
