! ------------------------------------------------------------------------
!>  \brief Module for Body Element Integration
! ------------------------------------------------------------------------
MODULE body_element_integration
USE pde_solver_constants          !> Module for PDE Solver constants (for Gaussian quadrature)
USE constitutive                  !> Constitutive Matrix module
USE body_element_interpolation    !> Body Element Interpolation module (shape functions)
USE dense_matrix_def              !> Dense Matrix data type
USE field_data                    !> Field Data module
USE kinematic                     !> Kinematic Matrix module
USE material_data                 !> Material Property Data module
USE vector_def                    !> Vector data type

IMPLICIT NONE
PRIVATE



! ************************************************************************
! EXPORTS
! ************************************************************************


!> Exported interfaces
PUBLIC :: bint_emass, bint_estiff, bint_eacc, bint_estress, bint_estrain



! ************************************************************************
! LOCAL CONSTANTS
! ************************************************************************

! none


! ************************************************************************
! DATA TYPES
! ************************************************************************

! none



! ************************************************************************
! INTERFACES
! ************************************************************************


!> \brief Interface to element mass matrix
INTERFACE bint_emass
  MODULE PROCEDURE bint_emass_3_noded_triangular_
END INTERFACE bint_emass

!> \brief Interface to element stiffness matrix
INTERFACE bint_estiff
  MODULE PROCEDURE bint_estiff_3_noded_triangular_
END INTERFACE bint_estiff

!> \brief Interface to element force vector due to body acceleration
INTERFACE bint_eacc
  MODULE PROCEDURE bint_eacc_3_noded_triangular_
END INTERFACE bint_eacc

!> \brief Interface to element force vector due to initial stress
INTERFACE bint_estress
  MODULE PROCEDURE bint_estress_3_noded_triangular_
END INTERFACE bint_estress

!> \brief Interface to element force vector due to initial strain
INTERFACE bint_estrain
  MODULE PROCEDURE bint_estrain_3_noded_triangular_
END INTERFACE bint_estrain



CONTAINS



! ************************************************************************
! ACCESS PROGRAMS
! ************************************************************************


! ------------------------------------------------------------------------
!>  \brief Element mass matrix (3-noded triangular)
!!
!!  \param  iel   Element number
!!  \param  emass Element mass matrix
!!
!!  This routine performs the following integration:
!!
!!      \int_A N^T * rho * N dA
!!
!!  where N is the shape function matrix, rho is the material density,
!!  and A is the area of the element. The routine uses the technique of
!!  Gaussian quadrature to numerically evaluate the integral. However,
!!  note that the integration points and weights are selected such that
!!  the exact integral is obtained. Consequently, only the usual floating
!!  point error is present in the result of the integration.
! ------------------------------------------------------------------------
SUBROUTINE bint_emass_3_noded_triangular_ (iel, emass)
  INTEGER, INTENT(IN) :: iel
  TYPE(matrixT), INTENT(INOUT) :: emass
  TYPE(matrixT) :: N        !> shape function matrix
  DOUBLE PRECISION :: rho   !> material density
  DOUBLE PRECISION :: A     !> element area
  INTEGER :: m    !> material type
  INTEGER :: i    !> loop variable

  m = fld_getMaterial(iel)    !> get material type
  rho = mtl_getDens(m)        !> get density
  A = fld_volElem(iel)        !> get element area

  !> initialize element mass matrix
  CALL dm_init(emass, NNODEL*NDIM,NNODEL*NDIM)

  !> perform Gaussian quadrature
  DO i = 1,NGAUSS_ELEM

    !> get shape functions at integration point
    CALL bshp_shape(GAUSS_PT_ELEM(i,1),GAUSS_PT_ELEM(i,2), N)

    !> increment element mass matrix
    emass%dat = emass%dat &
                + GAUSS_WT_ELEM(i)*rho*A*MATMUL(TRANSPOSE(N%dat),N%dat)

  END DO

  !> deallocate shape function matrix
  CALL dm_clean(N)

END SUBROUTINE bint_emass_3_noded_triangular_


! ------------------------------------------------------------------------
!>  \brief Element stiffness matrix (3-noded triangular)
!!
!!  \param  iel     Element number
!!  \param  estiff  Element stiffness matrix
!!
!!  This routine performs the following integration:
!!
!!      \int_A B^T * D * B dA
!!
!!  where B is the kinematic matrix, D is the constitutive matrix,
!!  and A is the area of the element. Since B and D are constants for this
!!  type of element, the integration reduces to (B^T * D * B * A).
! ------------------------------------------------------------------------
SUBROUTINE bint_estiff_3_noded_triangular_ (iel, estiff)
  INTEGER, INTENT(IN) :: iel
  TYPE(matrixT), INTENT(INOUT) :: estiff
  TYPE(matrixT) :: B      !> kinematic matrix
  TYPE(matrixT) :: D      !> constitutive matrix
  DOUBLE PRECISION :: emod, nu    !> material properties
  DOUBLE PRECISION :: A           !> element area
  INTEGER :: m    !> material type

  m = fld_getMaterial(iel)    !> get material type
  emod = mtl_getEmod(m)       !> get elastic modulus
  nu = mtl_getPois(m)         !> get Poisson's ratio
  A = fld_volElem(iel)        !> get element area

  !> initialize element stiffness matrix
  CALL dm_init(estiff, NNODEL*NDIM,NNODEL*NDIM)

  !> get kinematic and constitutive matrices
  CALL bmatrix(iel, B)
  CALL dmatrix(emod,nu, D)

  !> compute element stiffness matrix
  estiff%dat = A * MATMUL(TRANSPOSE(B%dat),MATMUL(D%dat,B%dat))

  !> deallocate kinematic and constitutive matrices
  CALL dm_clean(B)
  CALL dm_clean(D)

END SUBROUTINE bint_estiff_3_noded_triangular_


! ------------------------------------------------------------------------
!>  \brief Element force vector due to body acceleration (3-noded triangular)
!!
!!  \param  iel     Element number
!!  \param  eload   Element load vector
!!
!!  This routine performs the following integration:
!!
!!      \int_A N^T * rho * f dA
!!
!!  where N is the shape function matrix, rho is the material density,
!!  f is the element body acceleration vector, and A is the area of the
!!  element. The routine uses the technique of Gaussian quadrature to
!!  numerically evaluate the integral. However, note that the integration
!!  points and weights are selected such that the exact integral is
!!  obtained. Consequently, only the usual floating point error is present
!!  in the result of the integration.
! ------------------------------------------------------------------------
SUBROUTINE bint_eacc_3_noded_triangular_ (iel, eload)
  INTEGER, INTENT(IN) :: iel
  TYPE(vectorT), INTENT(INOUT) :: eload
  TYPE(matrixT) :: N        !> shape function matrix
  TYPE(vectorT) :: f        !> body acceleration vector
  DOUBLE PRECISION :: rho   !> density
  DOUBLE PRECISION :: A     !> element area
  INTEGER :: m    !> material type
  INTEGER :: i,j  !> loop variables

  !> initialize body acceleration vector
  CALL vec_init(f, NNODEL*NDIM)

  !> load body acceleration at nodes
  DO i = 1,NNODEL
    DO j = 1,NDIM
      CALL vec_set( f, (i-1)*NDIM+j, fld_getBodyAcc( fld_getConnect(iel,i), j ) )
    END DO
  END DO

  m = fld_getMaterial(iel)    !> get material type
  rho = mtl_getDens(m)        !> get density
  A = fld_volElem(iel)        !> get element area

  !> initialize element load vector
  CALL vec_init(eload, NNODEL*NDIM)

  !> perform Gaussian quadrature
  DO i = 1,NGAUSS_ELEM

    !> get shape functions at integration point
    CALL bshp_shape(GAUSS_PT_ELEM(i,1),GAUSS_PT_ELEM(i,2), N)

    !> increment element load vector
    eload%dat = eload%dat &
                + GAUSS_WT_ELEM(i)*rho*A*MATMUL(TRANSPOSE(N%dat),f%dat)

  END DO

  !> deallocate objects
  CALL vec_clean(f)
  CALL dm_clean(N)

END SUBROUTINE bint_eacc_3_noded_triangular_


! ------------------------------------------------------------------------
!>  \brief Element force vector due to initial stress (3-noded triangular)
!!
!!  \param  iel     Element number
!!  \param  eload   Element load vector
!!
!!  This routine performs the following integration:
!!
!!      \int_A B^T * sig0 dA
!!
!!  where B is the kinematic matrix, sig0 is the initial stress vector,
!!  and A is the area of the element. Since B and sig0 are constant for
!!  the 3-noded triangular element, the integration reduces to
!!  (B^T * sig0 * A).
! ------------------------------------------------------------------------
SUBROUTINE bint_estress_3_noded_triangular_ (iel, eload)
  INTEGER, INTENT(IN) :: iel
  TYPE(vectorT), INTENT(INOUT) :: eload
  TYPE(matrixT) :: B        !> kinematic matrix
  TYPE(vectorT) :: sig0     !> initial stress vector
  DOUBLE PRECISION :: A     !> element area

  !> initialize initial stress vector
  CALL vec_init(sig0, NTNS)

  !> load initial stress vector
  CALL vec_set( sig0, 1, fld_getStressElem(iel,s11) )
  CALL vec_set( sig0, 2, fld_getStressElem(iel,s22) )
  CALL vec_set( sig0, 3, fld_getStressElem(iel,s12) )

  !> get element area
  A = fld_volElem(iel)

  !> get kinematic matrix
  CALL bmatrix(iel,B)

  !> initialize element load vector
  CALL vec_init(eload, NNODEL*NDIM)

  !> compute element load vector
  eload%dat = A * MATMUL(TRANSPOSE(B%dat),sig0%dat)

  !> deallocate objects
  CALL vec_clean(sig0)
  CALL dm_clean(B)

END SUBROUTINE bint_estress_3_noded_triangular_


! ------------------------------------------------------------------------
!>  \brief Element force vector due to initial strain (3-noded triangular)
!!
!!  \param  iel     Element number
!!  \param  eload   Element load vector
!!
!!  This routine performs the following integration:
!!
!!      \int_A B^T * D * eps0 dA
!!
!!  where B is the kinematic matrix, D is the constitutive matrix,
!!  eps0 is the initial strain vector, and A is the area of the element.
!!  Since B, D, and eps0 are constant for the 3-noded triangular element,
!!  the integration reduces to (B^T * D * eps0 * A).
! ------------------------------------------------------------------------
SUBROUTINE bint_estrain_3_noded_triangular_ (iel, eload)
  INTEGER, INTENT(IN) :: iel
  TYPE(vectorT), INTENT(INOUT) :: eload
  TYPE(matrixT) :: B        !> kinematic matrix
  TYPE(matrixT) :: D        !> constitutive matrix
  TYPE(vectorT) :: eps0     !> initial strain vector
  INTEGER :: m                  !> material number
  DOUBLE PRECISION :: emod, nu  !> material properties
  DOUBLE PRECISION :: A         !> element area

  !> initialize initial strain vector
  CALL vec_init(eps0, NTNS)

  !> load initial strain vector
  CALL vec_set( eps0, 1, fld_getStrainElem(iel,s11) )
  CALL vec_set( eps0, 2, fld_getStrainElem(iel,s22) )
  CALL vec_set( eps0, 3, fld_getStrainElem(iel,s12) )

  !> get element area
  A = fld_volElem(iel)

  !> get kinematic matrix
  CALL bmatrix(iel,B)

  !> get material properties
  m = fld_getMaterial(iel)
  emod = mtl_getEmod(m)
  nu = mtl_getPois(m)

  !> get constitutive matrix
  CALL dmatrix(emod,nu, D)

  !> initialize element load vector
  CALL vec_init(eload, NNODEL*NDIM)

  !> compute element load vector
  eload%dat = A * MATMUL( TRANSPOSE(B%dat), MATMUL(D%dat, eps0%dat) )

  !> deallocate objects
  CALL vec_clean(eps0)
  CALL dm_clean(B)
  CALL dm_clean(D)

END SUBROUTINE bint_estrain_3_noded_triangular_



! ************************************************************************
! LOCAL FUNCTIONS
! ************************************************************************

! none

END MODULE body_element_integration




! ------------------------------------------------------------------------
!>  \brief Module for Traction Element Integration
! ------------------------------------------------------------------------
MODULE traction_element_integration
USE pde_solver_constants              !>
USE traction_element_interpolation    !> traction element shape functions
USE dense_matrix_def                  !> Dense Matrix data type
USE vector_def                        !> Vector data type
USE boundary_data                     !> boundary element data

IMPLICIT NONE
PRIVATE



! ************************************************************************
! EXPORTS
! ************************************************************************


!> Exported interfaces
PUBLIC :: tint_etrac



! ************************************************************************
! LOCAL CONSTANTS
! ************************************************************************

! none


! ************************************************************************
! DATA TYPES
! ************************************************************************

! none



! ************************************************************************
! INTERFACES
! ************************************************************************


!> \brief Interface to element mass matrix
INTERFACE tint_etrac
  MODULE PROCEDURE tint_etrac_2_noded_linear_
END INTERFACE tint_etrac



CONTAINS



! ************************************************************************
! ACCESS PROGRAMS
! ************************************************************************


! ------------------------------------------------------------------------
!>  \brief Element load vector for traction element (2-noded linear)
!!
!!  \param  iel   Element number
!!  \param  eload Element load vector
!!
!!  This routine performs the following integration:
!!
!!      \int_0^1 Nt^T * T^T * Nt * tt * lt ds
!!
!!  where Nt is the shape function matrix, T is the transformation matrix,
!!  tt is the local traction vector, and lt is the length of the element.
!!  The routine uses the technique of Gaussian quadrature to numerically
!!  evaluate the integral. However, note that the integration points and
!!  weights are selected such that the exact integral is obtained.
!!  Consequently, only the usual floating point error is present in the
!!  result of the integration.
! ------------------------------------------------------------------------
SUBROUTINE tint_etrac_2_noded_linear_ (iel, eload)
  INTEGER, INTENT(IN) :: iel
  TYPE(vectorT), INTENT(INOUT) :: eload
  TYPE(matrixT) :: Nt       !> shape function matrix
  TYPE(matrixT) :: T        !> transformation matrix
  TYPE(vectorT) :: tt       !> local traction vector
  TYPE(surfLoadT) :: t1,t2  !> nodal tractions
  DOUBLE PRECISION :: lt    !> element length
  INTEGER :: i    !> loop variable

  lt = bnd_lenBoundElem(iel)    !> get element length

  !> get transformation matrix
  CALL tshp_transform(iel, T)

  !> get nodal tractions
  t1 = bnd_getTrac(iel,1)
  t2 = bnd_getTrac(iel,2)

  !> set up traction vector
  CALL vec_init(tt, NNODELB*NDIM)
  CALL vec_set(tt, 1, t1%sig_nt)
  CALL vec_set(tt, 2, t1%sig_nn)
  CALL vec_set(tt, 3, t2%sig_nt)
  CALL vec_set(tt, 4, t2%sig_nn)

  !> initialize element load vector
  CALL vec_init(eload, NNODELB*NDIM)

  !> perform Gaussian quadrature
  DO i = 1,NGAUSS_BOUND

    !> get shape functions at integration point
    CALL tshp_shape(GAUSS_PT_BOUND(i), Nt)

    !> increment element load vector
    eload%dat = eload%dat &
                + GAUSS_WT_BOUND(i)*lt &
                  * MATMUL(MATMUL(TRANSPOSE(Nt%dat),MATMUL(TRANSPOSE(T%dat),Nt%dat)),tt%dat)

  END DO

  !> deallocate objects
  CALL dm_clean(Nt)
  CALL dm_clean(T)
  CALL vec_clean(tt)

END SUBROUTINE tint_etrac_2_noded_linear_



! ************************************************************************
! LOCAL FUNCTIONS
! ************************************************************************

! none

END MODULE traction_element_integration
