! ------------------------------------------------------------------------
!>  \brief Module for testing PDE Solver Control module
! ------------------------------------------------------------------------
MODULE pde_solver_control_test
USE fruit                   !> FRUIT unit testing framework for Fortran
USE system_constants        !> global system constants
USE pde_solver_constants    !> constants for PDE solver
USE log_message_control     !> print log/error messages
USE log_messages            !> log/error message and sender codes
USE band_sym_matrix_def     !> banded symmetric matrix ADT
USE dense_matrix_def        !> dense matrix ADT
USE vector_def              !> vector ADT
USE field_data              !> field data module
USE boundary_data           !> boundary data module
USE material_data           !> material property data module
USE pde_solver_control      !> PDE solver contorl module

IMPLICIT NONE


CONTAINS


! ------------------------------------------------------------------------
!>  \brief  Initialization test for PDE Solver Control
!!
!!  \param unit_name_size     Name of unit test (for FRUIT)
!!  \param unit_name_area     Name of unit test (for FRUIT)
!!  \param unit_name_mass     Name of unit test (for FRUIT)
!!  \param unit_name_stiff    Name of unit test (for FRUIT)
!!  \param unit_name_clean    Name of unit test (for FRUIT)
!!  \param testName           Filename for log file (required for exceptions)
!!  \param nnodExp            Expected number of nodes (15)
!!  \param nelExp             Expected number of elements (16)
!!  \param ndofExp            Expected number of degrees of freedom (24)
!!  \param hbwExp             Expected half bandwidth (10)
!!  \param ntnsExp            Expected number of tensor variables (48)
!!  \param nmtl               Number of material types (1)
!!  \param rho                Density of material 1 (2200 kg/m^3)
!!  \param emod               Elastic modulus of material 1 (26000 MPa)
!!  \param nu                 Poisson's ratio of material 1 (0.25)
!!  \param expArea            Expected total area (5.0 x 8.0 = 40.0)
!!  \param totalArea          Actual total area (from summing element areas)
!!  \param expMass            For checking mass matrix (see test_straight.xlsx)
!!  \param expStiff           For checking stiffness matrix (see test_straight.xlsx)
!!  \param toler              Tolerance for floating point error
!!  \param expMsg             Expected exception message
!!  \param actMsg             Actual exception message
!!
!!  This routine tests tests correct allocation and initialization of
!!  state variables for a simple system. The system is a straight column
!!  with a fixed base. It has a width of 5 m and a height of 8 m. The
!!  mesh of the system has the following properties:
!!
!!    nnod  = 15
!!    nel   = 16
!!    ndof  = 24
!!    hbw   = 10
!!
!!  These parameters are verified after the initialization. The test also
!!  verifies the size of all system matrices and vectors. The test also
!!  deallocates the system and verifies that everything is set back to
!!  zero post-deallocation.
! ------------------------------------------------------------------------
SUBROUTINE test_pde_solver_initialization
  CHARACTER (LEN=*), PARAMETER :: unit_name_size = 'test_pde_solver_initialization_size'
  CHARACTER (LEN=*), PARAMETER :: unit_name_area = 'test_pde_solver_initialization_area'
  CHARACTER (LEN=*), PARAMETER :: unit_name_mass = 'test_pde_solver_initialization_mass'
  CHARACTER (LEN=*), PARAMETER :: unit_name_stiff = 'test_pde_solver_initialization_stiff'
  CHARACTER (LEN=*), PARAMETER :: unit_name_clean = 'test_pde_solver_initialization_clean'
  CHARACTER (LEN=*), PARAMETER :: testName = 'test_straight_coarse'
  INTEGER, PARAMETER :: nnodExp = 15
  INTEGER, PARAMETER :: nelExp = 16
  INTEGER, PARAMETER :: ndofExp = 24
  INTEGER, PARAMETER :: hbwExp = 10
  INTEGER, PARAMETER :: ntnsExp = nelExp*NTNS
  INTEGER, PARAMETER :: nmtl = 1
  DOUBLE PRECISION, PARAMETER :: rho = 2200.d0
  DOUBLE PRECISION, PARAMETER :: emod = 26.d9
  DOUBLE PRECISION, PARAMETER :: nu = 0.25d0
  DOUBLE PRECISION, PARAMETER :: expArea = 40.d0
  DOUBLE PRECISION :: totalArea
  DOUBLE PRECISION, PARAMETER :: toler = 1.d-12
  DOUBLE PRECISION :: expMass(hbwExp,ndofExp)
  DOUBLE PRECISION :: expStiff(hbwExp,ndofExp)
  INTEGER, PARAMETER :: expMsg=OK
  INTEGER :: actMsg
  INTEGER :: i,j    !> loop variables
  DOUBLE PRECISION :: ycoord    !> for setting up coordinates

  !> initialize log message file
  CALL log_setFileName(testName)
  CALL log_initLogFile()

  !> initialize material data
  CALL mtl_init(nmtl)
  CALL mtl_setDens(1,rho)
  CALL mtl_setEmod(1,emod)
  CALL mtl_setPois(1,nu)

  !> initialize node data
  CALL fld_initNode(nnodExp)

  !> x-coords of left column of nodes = 0

  !> x-coords of middle column of nodes = 2.5
  DO i = 2,nnodExp,3
    CALL fld_setCoord(i,1,2.5d0)
  END DO

  !> x-coords of right column of nodes = 5
  DO i = 3,nnodExp,3
    CALL fld_setCoord(i,1,5.d0)
  END DO

  !> y-coords, steps of 2
  DO i = 1,nnodExp,3
    ycoord = ONE_THIRD*DBLE(i-1)*2.d0
    CALL fld_setCoord(i,2,ycoord)
    CALL fld_setCoord(i+1,2,ycoord)
    CALL fld_setCoord(i+2,2,ycoord)
  END DO

  !> fix base
  CALL fld_setFix(1,1,.TRUE.)
  CALL fld_setFix(1,2,.TRUE.)
  CALL fld_setFix(2,1,.TRUE.)
  CALL fld_setFix(2,2,.TRUE.)
  CALL fld_setFix(3,1,.TRUE.)
  CALL fld_setFix(3,2,.TRUE.)

  !> initialize system degrees of freedom
  CALL fld_initDof()


  !> initialize element data
  CALL fld_initElem(nelExp)

  !> element 1
  CALL fld_setConnect(1,1,1)
  CALL fld_setConnect(1,2,2)
  CALL fld_setConnect(1,3,4)

  !> element 2
  CALL fld_setConnect(2,1,2)
  CALL fld_setConnect(2,2,5)
  CALL fld_setConnect(2,3,4)

  !> element 3
  CALL fld_setConnect(3,1,2)
  CALL fld_setConnect(3,2,6)
  CALL fld_setConnect(3,3,5)

  !> element 4
  CALL fld_setConnect(4,1,2)
  CALL fld_setConnect(4,2,3)
  CALL fld_setConnect(4,3,6)

  !> element 5
  CALL fld_setConnect(5,1,4)
  CALL fld_setConnect(5,2,5)
  CALL fld_setConnect(5,3,7)

  !> element 6
  CALL fld_setConnect(6,1,5)
  CALL fld_setConnect(6,2,8)
  CALL fld_setConnect(6,3,7)

  !> element 7
  CALL fld_setConnect(7,1,5)
  CALL fld_setConnect(7,2,9)
  CALL fld_setConnect(7,3,8)

  !> element 8
  CALL fld_setConnect(8,1,5)
  CALL fld_setConnect(8,2,6)
  CALL fld_setConnect(8,3,9)

  !> element 9
  CALL fld_setConnect(9,1,7)
  CALL fld_setConnect(9,2,8)
  CALL fld_setConnect(9,3,10)

  !> element 10
  CALL fld_setConnect(10,1,8)
  CALL fld_setConnect(10,2,11)
  CALL fld_setConnect(10,3,10)

  !> element 11
  CALL fld_setConnect(11,1,8)
  CALL fld_setConnect(11,2,12)
  CALL fld_setConnect(11,3,11)

  !> element 12
  CALL fld_setConnect(12,1,8)
  CALL fld_setConnect(12,2,9)
  CALL fld_setConnect(12,3,12)

  !> element 13
  CALL fld_setConnect(13,1,10)
  CALL fld_setConnect(13,2,11)
  CALL fld_setConnect(13,3,13)

  !> element 14
  CALL fld_setConnect(14,1,11)
  CALL fld_setConnect(14,2,14)
  CALL fld_setConnect(14,3,13)

  !> element 15
  CALL fld_setConnect(15,1,11)
  CALL fld_setConnect(15,2,15)
  CALL fld_setConnect(15,3,14)

  !> element 16
  CALL fld_setConnect(16,1,11)
  CALL fld_setConnect(16,2,12)
  CALL fld_setConnect(16,3,15)


  !> set element materials
  DO i = 1,nelExp
    CALL fld_setMaterial(i,1)
  END DO


  !> initialize unit test
  CALL set_unit_name(unit_name_size)

  !> initialize pde solver
  CALL pde_init(actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> check system parameters
  CALL assertEquals(nnodExp,nnod)
  CALL assertEquals(nelExp,nel)
  CALL assertEquals(ndofExp,ndof)
  CALL assertEquals(hbwExp,hbw)

  !> check state variables (matrices and vectors)
  CALL assertEquals(hbwExp,bsm_halfBW(mass))
  CALL assertEquals(ndofExp,bsm_numRows(mass))
  CALL assertEquals(hbwExp,bsm_halfBW(stiff))
  CALL assertEquals(ndofExp,bsm_numRows(stiff))
  CALL assertEquals(ndofExp,vec_length(initStress))
  CALL assertEquals(ndofExp,vec_length(initStrain))
  CALL assertEquals(ndofExp,vec_length(body))
  CALL assertEquals(ndofExp,vec_length(trac))
  CALL assertEquals(ndofExp,vec_length(load))
  CALL assertEquals(ndofExp,vec_length(prevDisp))
  CALL assertEquals(ndofExp,vec_length(incDisp))
  CALL assertEquals(ndofExp,vec_length(newDisp))
  CALL assertEquals(ndofExp,vec_length(prevVel))
  CALL assertEquals(ndofExp,vec_length(incVel))
  CALL assertEquals(ndofExp,vec_length(newVel))
  CALL assertEquals(ndofExp,vec_length(prevAcc))
  CALL assertEquals(ndofExp,vec_length(newAcc))
  CALL assertEquals(ntnsExp,vec_length(prevStress))
  CALL assertEquals(ntnsExp,vec_length(incStress))
  CALL assertEquals(ntnsExp,vec_length(newStress))
  CALL assertEquals(ntnsExp,vec_length(prevStrain))
  CALL assertEquals(ntnsExp,vec_length(incStrain))
  CALL assertEquals(ntnsExp,vec_length(newStrain))


  !> initialize unit test
  CALL set_unit_name(unit_name_area)

  !> check total area
  totalArea = 0.d0
  DO i = 1,nel
    totalArea = totalArea + fld_volElem(i)
  END DO
  CALL assertEquals(expArea,totalArea,toler)


  !> initialize unit test
  CALL set_unit_name(unit_name_mass)

  !> set up expected mass matrix data
  expMass = RESHAPE( (/ &
                      0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
                      0.d0, 1833.33333333333333d0, 0.d0, 0.d0, 0.d0, &
                      0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
                      1833.33333333333333d0, 0.d0, 0.d0, 0.d0, 0.d0, &
                      0.d0, 0.d0, 0.d0, 1222.22222222222222d0, 0.d0, &
                      3666.66666666666667d0, 0.d0, 0.d0, 0.d0, 0.d0, &
                      0.d0, 0.d0, 0.d0, 1222.22222222222222d0, 0.d0, &
                      3666.66666666666667d0, 0.d0, 0.d0, 0.d0, 0.d0, &
                      0.d0, 0.d0, 0.d0, 1222.22222222222222d0, 0.d0, &
                      1833.33333333333333d0, 0.d0, 0.d0, 0.d0, 0.d0, &
                      0.d0, 0.d0, 0.d0, 1222.22222222222222d0, 0.d0, &
                      1833.33333333333333d0, 0.d0, 0.d0, 0.d0, &
                      611.111111111111111d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, 0.d0, 0.d0, &
                      1833.33333333333333d0, 0.d0, 0.d0, 0.d0, &
                      611.111111111111111d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, 0.d0, 0.d0, &
                      1833.33333333333333d0, 0.d0, 0.d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, 0.d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, &
                      3666.66666666666667d0, 0.d0, 0.d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, 0.d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, &
                      3666.66666666666667d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, &
                      611.111111111111111d0, 0.d0, 0.d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, &
                      1833.33333333333333d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, &
                      611.111111111111111d0, 0.d0, 0.d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, &
                      1833.33333333333333d0, 0.d0, 0.d0, 0.d0, &
                      611.111111111111111d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, 0.d0, 0.d0, &
                      1833.33333333333333d0, 0.d0, 0.d0, 0.d0, &
                      611.111111111111111d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, 0.d0, 0.d0, &
                      1833.33333333333333d0, 0.d0, 0.d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, 0.d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, &
                      3666.66666666666667d0, 0.d0, 0.d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, 0.d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, &
                      3666.66666666666667d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, &
                      611.111111111111111d0, 0.d0, 0.d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, &
                      1833.33333333333333d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, &
                      611.111111111111111d0, 0.d0, 0.d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, &
                      1833.33333333333333d0, 0.d0, 0.d0, 0.d0, &
                      611.111111111111111d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, 0.d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, 0.d0, 0.d0, &
                      611.111111111111111d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, 0.d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, 0.d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, 0.d0, 0.d0, &
                      611.111111111111111d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, 0.d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, 0.d0, 0.d0, &
                      611.111111111111111d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, &
                      611.111111111111111d0, 0.d0, 0.d0, 0.d0, &
                      611.111111111111111d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, &
                      1222.22222222222222d0, 0.d0, &
                      611.111111111111111d0, 0.d0, 0.d0, 0.d0, &
                      611.111111111111111d0, 0.d0, &
                      1222.22222222222222d0 &
                      /), SHAPE(expMass) )

  !> set actual mass matrix
  CALL pde_buildMassMatrix(actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> check mass matrix
  CALL assertEquals(expMass,mass%dat, hbwExp,ndofExp, toler)


  !> initialize unit test
  CALL set_unit_name(unit_name_stiff)

  !> set up expected stiffness matrix data
  expStiff = RESHAPE( (/ &
                      0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
                      0.d0, 37960000000.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
                      0.d0, 0.d0, 0.d0, 0.d0, 10400000000.d0, &
                      47320000000.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
                      0.d0, 0.d0, -24960000000.d0, -10400000000.d0, &
                      75920000000.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
                      0.d0, -10400000000.d0, -8320000000.d0, 0.d0, &
                      94640000000.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
                      0.d0, 0.d0, -24960000000.d0, 10400000000.d0, &
                      37960000000.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
                      0.d0, 10400000000.d0, -8320000000.d0, &
                      -10400000000.d0, 47320000000.d0, 0.d0, 0.d0, &
                      0.d0, -6500000000.d0, -5200000000.d0, 0.d0, &
                      10400000000.d0, 0.d0, 0.d0, 37960000000.d0, &
                      0.d0, 0.d0, -5200000000.d0, -19500000000.d0, &
                      10400000000.d0, 0.d0, 0.d0, 0.d0, 10400000000.d0, &
                      47320000000.d0, 0.d0, 0.d0, 0.d0, -13000000000.d0, &
                      0.d0, 0.d0, 0.d0, -24960000000.d0, &
                      -10400000000.d0, 75920000000.d0, 0.d0, 0.d0, 0.d0, &
                      -39000000000.d0, 0.d0, 0.d0, -10400000000.d0, &
                      -8320000000.d0, 0.d0, 94640000000.d0, 0.d0, 0.d0, &
                      -10400000000.d0, -6500000000.d0, 5200000000.d0, &
                      0.d0, 0.d0, -24960000000.d0, 10400000000.d0, &
                      37960000000.d0, -10400000000.d0, 0.d0, &
                      5200000000.d0, -19500000000.d0, 0.d0, 0.d0, &
                      10400000000.d0, -8320000000.d0, -10400000000.d0, &
                      47320000000.d0, 0.d0, 0.d0, 0.d0, -6500000000.d0, &
                      -5200000000.d0, 0.d0, 10400000000.d0, 0.d0, 0.d0, &
                      37960000000.d0, 0.d0, 0.d0, -5200000000.d0, &
                      -19500000000.d0, 10400000000.d0, 0.d0, 0.d0, 0.d0, &
                      10400000000.d0, 47320000000.d0, 0.d0, 0.d0, 0.d0, &
                      -13000000000.d0, 0.d0, 0.d0, 0.d0, &
                      -24960000000.d0, -10400000000.d0, 75920000000.d0, &
                      0.d0, 0.d0, 0.d0, -39000000000.d0, 0.d0, 0.d0, &
                      -10400000000.d0, -8320000000.d0, 0.d0, &
                      94640000000.d0, 0.d0, 0.d0, -10400000000.d0, &
                      -6500000000.d0, 5200000000.d0, 0.d0, 0.d0, &
                      -24960000000.d0, 10400000000.d0, 37960000000.d0, &
                      -10400000000.d0, 0.d0, 5200000000.d0, &
                      -19500000000.d0, 0.d0, 0.d0, 10400000000.d0, &
                      -8320000000.d0, -10400000000.d0, 47320000000.d0, &
                      0.d0, 0.d0, 0.d0, -6500000000.d0, -5200000000.d0, &
                      0.d0, 10400000000.d0, 0.d0, 0.d0, 18980000000.d0, &
                      0.d0, 0.d0, -5200000000.d0, -19500000000.d0, &
                      10400000000.d0, 0.d0, 0.d0, 0.d0, 0.d0, &
                      23660000000.d0, 0.d0, 0.d0, 0.d0, -13000000000.d0, &
                      0.d0, 0.d0, 0.d0, -12480000000.d0, -5200000000.d0, &
                      37960000000.d0, 0.d0, 0.d0, 0.d0, -39000000000.d0, &
                      0.d0, 0.d0, -5200000000.d0, -4160000000.d0, 0.d0, &
                      47320000000.d0, 0.d0, 0.d0, -10400000000.d0, &
                      -6500000000.d0, 5200000000.d0, 0.d0, 0.d0, &
                      -12480000000.d0, 5200000000.d0, 18980000000.d0, &
                      -10400000000.d0, 0.d0, 5200000000.d0, &
                      -19500000000.d0, 0.d0, 0.d0, 5200000000.d0, &
                      -4160000000.d0, 0.d0, 23660000000.d0 &
                      /), SHAPE(expStiff) )

  !> set actual stiffness matrix
  CALL pde_buildStiffMatrix(actMsg)
  CALL assertEquals(expMsg,actMsg)

  !> check stiffness matrix
  CALL assertEquals(expStiff,stiff%dat, hbwExp,ndofExp, toler)


  !> initialize unit test
  CALL set_unit_name(unit_name_clean)

  !> finalize pde solver
  CALL pde_clean()

  !> check system parameters
  CALL assertEquals(0,nnod)
  CALL assertEquals(0,nel)
  CALL assertEquals(0,ndof)
  CALL assertEquals(0,hbw)

  !> check state variables (matrices and vectors)
  CALL assertEquals(0,bsm_halfBW(mass))
  CALL assertEquals(0,bsm_numRows(mass))
  CALL assertEquals(0,bsm_halfBW(stiff))
  CALL assertEquals(0,bsm_numRows(stiff))
  CALL assertEquals(0,vec_length(initStress))
  CALL assertEquals(0,vec_length(initStrain))
  CALL assertEquals(0,vec_length(body))
  CALL assertEquals(0,vec_length(trac))
  CALL assertEquals(0,vec_length(load))
  CALL assertEquals(0,vec_length(prevDisp))
  CALL assertEquals(0,vec_length(incDisp))
  CALL assertEquals(0,vec_length(newDisp))
  CALL assertEquals(0,vec_length(prevVel))
  CALL assertEquals(0,vec_length(incVel))
  CALL assertEquals(0,vec_length(newVel))
  CALL assertEquals(0,vec_length(prevAcc))
  CALL assertEquals(0,vec_length(newAcc))
  CALL assertEquals(0,vec_length(prevStress))
  CALL assertEquals(0,vec_length(incStress))
  CALL assertEquals(0,vec_length(newStress))
  CALL assertEquals(0,vec_length(prevStrain))
  CALL assertEquals(0,vec_length(incStrain))
  CALL assertEquals(0,vec_length(newStrain))

  !> finalize log file and deallocate objects
  CALL log_closeLogFile()
  CALL fld_cleanNode()
  CALL fld_cleanElem()

END SUBROUTINE test_pde_solver_initialization

END MODULE pde_solver_control_test
